@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CatJob.cc,v 1.25 2004/09/22 13:31:31 lav Exp $ */
+/* $Id: CatJob.cc,v 1.26 2005/11/29 07:07:55 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -101,7 +101,7 @@ CatJob::CatJob(FileAccess *new_session,OutputJob *_output,ArgV *new_args)
    {
       const char *pager=getenv("PAGER");
       if(pager==NULL)
-	 pager="exec more";
+	 pager=DEFAULT_PAGER;
       output->PreFilter(pager);
    }
    if(!strcmp(op,"zcat") || !strcmp(op,"zmore"))
