@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CopyJob.h,v 1.13 2000/07/19 07:24:15 lav Exp $ */
+/* $Id: CopyJob.h,v 1.20 2005/06/22 09:26:00 lav Exp $ */
 
 #ifndef COPYJOB_H
 #define COPYJOB_H
@@ -32,21 +32,29 @@ class CopyJob : public Job
    FileCopy *c;
    bool done;
    char *name; // file name
+   char *dispname; // displayed file name
    char *op;   // command name
    bool no_status;
+   bool no_status_on_write;
+   bool clear_status_on_write;
+
+   void SetDispName();
 
 public:
    CopyJob(FileCopy *c1,const char *n,const char *op1);
    ~CopyJob();
 
    void NoStatus() { no_status=true; }
+   void NoStatusOnWrite() { no_status_on_write=true; }
+   void ClearStatusOnWrite() { clear_status_on_write=true; }
+   bool HasStatus() const { return !no_status; }
 
    int Do();
    int Done();
    int ExitCode();
 
-   void Suspend() { c->Suspend(); Job::Suspend(); }
-   void Resume() { Job::Resume(); c->Resume(); }
+   void SuspendInternal() { c->SuspendSlave(); }
+   void ResumeInternal()  { c->ResumeSlave(); }
    void Fg() { c->Fg(); Job::Fg(); }
    void Bg() { Job::Bg(); c->Bg(); }
 
@@ -55,8 +63,7 @@ public:
 
    bool Error() { return c->Error(); }
    const char *ErrorText() { return c->ErrorText(); }
-   long GetTimeSpent() { return c->GetTimeSpent(); }
-   int  GetTimeSpentMilli() { return c->GetTimeSpentMilli(); }
+   double GetTimeSpent() { return c->GetTimeSpent(); }
    off_t GetBytesCount() { return c->GetBytesCount(); }
    off_t GetSize() { return c->GetSize(); }
    off_t GetPos()  { return c->GetPos(); }
@@ -67,19 +74,22 @@ public:
    void SetRange(off_t s,off_t lim) { c->SetRange(s,lim); }
    void SetDate(time_t d) { c->SetDate(d); }
    void SetSize(off_t s)   { c->SetSize(s); }
+   FileCopy *GetCopy() { return c; }
    FileCopyPeer *GetPut() { return c->put; }
+   FileCopyPeer *GetGet() { return c->get; }
 
+   const char *Status(const StatusLine *s,bool base=false);
    void ShowRunStatus(StatusLine *s);
-   void	PrintStatus(int);
+   void	PrintStatus(int,const char *);
 
-   const char *GetName() { return name; }
-   const char *SqueezeName(int w);
+   const char *GetName() const { return name; }
+   const char *GetDispName() const { return dispname; }
+   const char *SqueezeName(int w, bool base=false);
 
    static CopyJob *NewGet(FileAccess *f,const char *src,const char *dst);
    static CopyJob *NewPut(FileAccess *f,const char *src,const char *dst);
-   static CopyJob *NewEcho(const char *str,int len,FDStream *o,const char *op);
-   static CopyJob *NewEcho(const char *str,FDStream *o,const char *op)
-      { return NewEcho(str,strlen(str),o,op); }
+
+   static const char *FormatBytesTimeRate(off_t bytes,double time);
 };
 
 class ArgV;
@@ -91,7 +101,7 @@ protected:
    int errors;
    int count;
    off_t bytes;
-   float time_spent;
+   double time_spent;
    const char *op;
    bool no_status;
    char *cwd;
@@ -107,7 +117,7 @@ protected:
 public:
    int Do();
    int Done();
-   int ExitCode() { return errors!=0; }
+   virtual int ExitCode() { return errors!=0; }
 
    int AcceptSig(int sig);
 
@@ -116,9 +126,12 @@ public:
 
    void SayFinalWithPrefix(const char *p);
    void SayFinal() { SayFinalWithPrefix(""); }
-   void	PrintStatus(int);
+   void	PrintStatus(int,const char *);
 
    void Ascii() { ascii=true; }
+
+   double GetTimeSpent() { return time_spent; }
+   off_t GetBytesCount() { return bytes; }
 };
 
 #endif // COPYJOB_H
