@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileSet.cc,v 1.46 2005/02/08 11:03:05 lav Exp $ */
+/* $Id: FileSet.cc,v 1.49 2005/12/31 07:01:35 lav Exp $ */
 
 #include <config.h>
 #include "FileSet.h"
@@ -326,6 +326,34 @@ void FileSet::SubtractOlderThan(time_t t)
       }
    }
 }
+void FileSet::SubtractNewerThan(time_t t)
+{
+   for(int i=0; i<fnum; i++)
+   {
+      if(files[i]->defined&FileInfo::TYPE
+      && files[i]->filetype!=FileInfo::NORMAL)
+	 continue;
+      if(files[i]->NewerThan(t))
+      {
+	 Sub(i);
+	 i--;
+      }
+   }
+}
+void FileSet::SubtractSizeOutside(const Range *r)
+{
+   for(int i=0; i<fnum; i++)
+   {
+      if(files[i]->defined&FileInfo::TYPE
+      && files[i]->filetype!=FileInfo::NORMAL)
+	 continue;
+      if(files[i]->SizeOutside(r))
+      {
+	 Sub(i);
+	 i--;
+      }
+   }
+}
 void FileSet::SubtractDirs()
 {
    for(int i=0; i<fnum; i++)
@@ -402,7 +430,15 @@ bool  FileInfo::SameAs(const FileInfo *fi,int ignore)
 
 bool  FileInfo::OlderThan(time_t t)
 {
-   return ((defined&DATE) && date<t);
+   return((defined&DATE) && date<t);
+}
+bool  FileInfo::NewerThan(time_t t)
+{
+   return((defined&DATE) && date>=t);
+}
+bool  FileInfo::SizeOutside(const Range *r)
+{
+   return((defined&SIZE) && !r->Match(size));
 }
 
 void FileSet::Count(int *d,int *f,int *s,int *o)
@@ -603,7 +639,10 @@ void FileSet::LocalChown(const char *dir)
 	       new_gid=g;
 	 }
 	 if(new_uid!=st.st_uid || new_gid!=st.st_gid)
-	    lchown(local_name,new_uid,new_gid);
+	 {
+	    if(lchown(local_name,new_uid,new_gid)==-1)
+	       /* don't care */;
+	 }
       }
    }
 }
@@ -831,7 +870,8 @@ FileInfo *FileInfo::parse_ls_line(const char *line_c,const char *tz)
    int prec=30;
    if(strlen(t)==5)
    {
-      sscanf(t,"%2d:%2d",&date.tm_hour,&date.tm_min);
+      if(sscanf(t,"%2d:%2d",&date.tm_hour,&date.tm_min)!=2)
+	 ERR;
       date.tm_year=guess_year(date.tm_mon,date.tm_mday,date.tm_hour,date.tm_min) - 1900;
    }
    else
