@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileSet.h,v 1.34 2005/02/08 11:03:05 lav Exp $ */
+/* $Id: FileSet.h,v 1.35 2005/11/21 05:47:27 lav Exp $ */
 
 #ifndef FILESET_H
 #define FILESET_H
@@ -29,6 +29,7 @@
 #undef TYPE
 
 class TimeInterval;
+class Range;
 
 class FileInfo
 {
@@ -92,6 +93,8 @@ public:
 
    bool	 SameAs(const FileInfo *,int ignore);
    bool	 OlderThan(time_t t);
+   bool	 NewerThan(time_t t);
+   bool  SizeOutside(const Range *r);
 
    void	 SetAssociatedData(void *d,int len)
       {
@@ -150,6 +153,8 @@ public:
    void	 SubtractSame(const FileSet *,int ignore);
    void	 SubtractAny(const FileSet *);
    void  SubtractOlderThan(time_t t);
+   void  SubtractNewerThan(time_t t);
+   void  SubtractSizeOutside(const Range *r);
    void  SubtractDirs();
    void  SubtractNotDirs();
    void  SubtractNotIn(const FileSet *);
