@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,28 +46,44 @@ FDStream::FDStream(int new_fd,const char *new_name)
       name=xstrdup(expand_home_relative(new_name));
    else
       name=0;
+   full_name=0;
+   cwd=0;
    error_text=0;
    status=0;
+   close_fd=false;
 }
 FDStream::FDStream()
 {
    fd=-1;
    name=0;
+   full_name=0;
    error_text=0;
    status=0;
+   close_fd=false;
 }
-void FDStream::MakeErrorText()
+void FDStream::MakeErrorText(int e)
 {
-   if(NonFatalError(errno))
+   if(!e)
+      e=errno;
+   if(NonFatalError(e))
       return;  // not a serious error - can be retried
-   char *syserr=strerror(errno);
+   char *syserr=strerror(e);
    error_text=(char*)xmalloc(strlen(name)+strlen(syserr)+3);
    sprintf(error_text,"%s: %s",name,syserr);
 }
+void FDStream::SetCwd(const char *new_cwd)
+{
+   xfree(cwd);
+   cwd=xstrdup(new_cwd);
+}
 FDStream::~FDStream()
 {
-   // don't close fd
+   if(close_fd)
+      close(fd);
+   if(full_name!=name)
+      xfree(full_name);
    xfree(name);
+   xfree(cwd);
    xfree(error_text);
 };
 
@@ -97,8 +113,7 @@ void OutputFilter::Child(int *p)
 	 dup2(second_fd,1);
 	 close(second_fd);
       }
-      int fl;
-      fcntl(1,F_GETFL,&fl);
+      int fl=fcntl(1,F_GETFL);
       fcntl(1,F_SETFL,fl&~O_NONBLOCK);
    }
 }
@@ -117,8 +132,7 @@ void InputFilter::Child(int *p)
 	 dup2(second_fd,0);
 	 close(second_fd);
       }
-      int fl;
-      fcntl(0,F_GETFL,&fl);
+      int fl=fcntl(0,F_GETFL);
       fcntl(0,F_SETFL,fl&~O_NONBLOCK);
    }
 }
@@ -142,15 +156,13 @@ int OutputFilter::getfd()
    }
 
    int	 p[2];
-   char  s[256];
    pid_t pid;
 
    if(pipe(p)==-1)
    {
       if(NonFatalError(errno))
 	 return -1;
-      sprintf(s,_("pipe() failed: %s"),strerror(errno));
-      error_text=xstrdup(s);
+      error_text=xasprintf(_("pipe() failed: %s"),strerror(errno));
       return -1;
    }
 
@@ -168,19 +180,36 @@ int OutputFilter::getfd()
       Child(p);
       if(stderr_to_stdout)
 	 dup2(1,2);
-      if(oldcwd)
+      if(stdout_to_null)
       {
-	 if(chdir(oldcwd)==-1)
+	 close(1);
+	 int null=open("/dev/null",O_RDWR);
+	 if(null==-1)
+	    perror("open(\"/dev/null\")");
+	 else if(null==0) {
+	    if(dup(0)==-1)
+	       perror("dup");
+	 }
+      }
+      if(cwd)
+      {
+	 if(chdir(cwd)==-1)
 	 {
-	    fprintf(stderr,_("chdir(%s) failed: %s\n"),oldcwd,strerror(errno));
+	    fprintf(stderr,_("chdir(%s) failed: %s\n"),cwd,strerror(errno));
 	    fflush(stderr);
 	    _exit(1);
 	 }
       }
       if(a)
-	 execvp(a->a0(),a->GetV());
-      execl("/bin/sh","sh","-c",name,NULL);
-      fprintf(stderr,_("execl(/bin/sh) failed: %s\n"),strerror(errno));
+      {
+	 execvp(a->a0(),a->GetVNonConst());
+	 fprintf(stderr,_("execvp(%s) failed: %s\n"),a->a0(),strerror(errno));
+      }
+      else
+      {
+	 execl("/bin/sh","sh","-c",name,(char*)NULL);
+	 fprintf(stderr,_("execl(/bin/sh) failed: %s\n"),strerror(errno));
+      }
       fflush(stderr);
       _exit(1);
    case(-1): /* error */
@@ -198,9 +227,6 @@ int OutputFilter::getfd()
    fcntl(fd,F_SETFD,FD_CLOEXEC);
    fcntl(fd,F_SETFL,O_NONBLOCK);
 
-   xfree(oldcwd);
-   oldcwd=0;
-
    // wait until the child stops.
    int info;
    waitpid(pid,&info,WUNTRACED);
@@ -220,19 +246,14 @@ void OutputFilter::Init()
    w=0;
    second=0;
    second_fd=-1;
-   oldcwd=xgetcwd();
+   cwd=xgetcwd();
    pg=0;
    closed=false;
    stderr_to_stdout=false;
+   stdout_to_null=false;
    delete_second=false;
 }
 
-void OutputFilter::SetCwd(const char *cwd)
-{
-   xfree(oldcwd);
-   oldcwd=xstrdup(cwd);
-}
-
 OutputFilter::OutputFilter(const char *filter,int new_second_fd)
    : FDStream(-1,filter)
 {
@@ -266,7 +287,7 @@ OutputFilter::OutputFilter(ArgV *a1,FDStream *new_second)
 
 OutputFilter::~OutputFilter()
 {
-   // `a' is not deleted here
+   delete a;
 
    close(fd);
    fd=-1;
@@ -274,8 +295,6 @@ OutputFilter::~OutputFilter()
    if(w)
       w->Auto();
 
-   xfree(oldcwd);
-
    if(delete_second)
       delete second;
 }
@@ -336,20 +355,16 @@ void FileStream::setmtime(time_t t)
 FileStream::FileStream(const char *fname,int new_mode) : FDStream(-1,fname)
 {
    mode=new_mode;
-   full_name=0;
    if(name[0]=='/')
       full_name=name;
    else
    {
-      char *cwd=xgetcwd();
+      cwd=xgetcwd();
       full_name=xstrdup(dir_file(cwd,name));
-      xfree(cwd);
    }
 }
 FileStream::~FileStream()
 {
-   if(full_name!=name)
-      xfree(full_name);
    if(fd!=-1)
       close(fd);
    fd=-1;
@@ -404,6 +419,12 @@ off_t FileStream::get_size()
 #include "SMTask.h"
 bool FDStream::NonFatalError(int err)
 {
+   if(err==EDQUOT || err==ENOSPC)
+   {
+      struct stat st;
+      if(fd>=0 && fstat(fd,&st)!=-1 && st.st_nlink==0)
+	 return false;
+   }
    bool non_fatal=SMTask::NonFatalError(err);
    if(non_fatal)
       set_status(strerror(err));
