@@ -49,20 +49,12 @@ void Fish::GetBetterConnection(int level)
       {
 	 if(level<2)
 	    continue;
-	 if(!connection_takeover || (o->priority>=priority && !o->suspended))
+	 if(!connection_takeover || (o->priority>=priority && !o->IsSuspended()))
 	    continue;
 	 o->Disconnect();
 	 return;
       }
 
-      if(home && !o->home)
-	 o->home=xstrdup(home);
-      else if(!home && o->home)
-	 home=xstrdup(o->home);
-
-      o->ExpandTildeInCWD();
-      ExpandTildeInCWD();
-
       if(level==0 && xstrcmp(real_cwd,o->real_cwd))
 	 continue;
 
@@ -204,7 +196,7 @@ int Fish::Do()
       Send("#VER 0.0.2\n"
 	   "echo '### 000'\n");
       PushExpect(EXPECT_VER);
-      if(home==0)
+      if(home_auto==0)
       {
 	 Send("#PWD\n"
 	      "pwd; echo '### 200'\n");
@@ -216,7 +208,7 @@ int Fish::Do()
    case CONNECTED:
       if(mode==CLOSED)
 	 return m;
-      if(home==0 && !RespQueueIsEmpty())
+      if(home.path==0 && !RespQueueIsEmpty())
 	 goto usual_return;
       ExpandTildeInCWD();
       if(mode!=CHANGE_DIR && xstrcmp(cwd,real_cwd))
@@ -224,7 +216,7 @@ int Fish::Do()
 	 if(path_queue_len==0 || strcmp(path_queue[path_queue_len-1],cwd))
 	 {
 	    Send("#CWD %s\n"
-		 "cd %s; echo '### 000'\n",cwd,shell_encode(cwd));
+		 "cd %s; echo '### 000'\n",cwd.path,shell_encode(cwd));
 	    PushExpect(EXPECT_CWD);
 	    PushDirectory(cwd);
 	 }
@@ -310,11 +302,14 @@ void Fish::MoveConnectionHere(Fish *o)
    RQ_alloc=o->RQ_alloc; o->RQ_alloc=0;
    RQ_head=o->RQ_head; o->RQ_head=0;
    RQ_tail=o->RQ_tail; o->RQ_tail=0;
-   event_time=o->event_time;
+   BumpEventTime(o->event_time);
    set_real_cwd(o->real_cwd);
    o->set_real_cwd(0);
    state=CONNECTED;
    o->Disconnect();
+   if(!home)
+      set_home(home_auto);
+   ResumeInternal();
 }
 
 void Fish::Disconnect()
@@ -331,6 +326,8 @@ void Fish::Disconnect()
       SetError(STORE_FAILED,0);
    received_greeting=false;
    password_sent=0;
+   xfree(home_auto); home_auto=0;
+   home_auto=xstrdup(FindHomeAuto());
 }
 
 void Fish::EmptyPathQueue()
@@ -345,7 +342,6 @@ void Fish::Init()
    state=DISCONNECTED;
    send_buf=0;
    recv_buf=0;
-   recv_buf_suspended=false;
    ssh=0;
    max_send=0;
    line=0;
@@ -708,8 +704,10 @@ int Fish::HandleReplies()
    recv_buf->Skip(s);
 
    int code=-1;
-   if(s>7 && !memcmp(line,"### ",4) && is_ascii_digit(line[4]))
-      sscanf(line+4,"%3d",&code);
+   if(s>7 && !memcmp(line,"### ",4)) {
+      if(sscanf(line+4,"%3d",&code)!=1)
+	 code=-1;
+   }
 
    DebugPrint("<--- ",line,ReplyLogPriority(code));
    if(code==-1)
@@ -752,7 +750,12 @@ int Fish::HandleReplies()
    case EXPECT_PWD:
       if(!message)
 	 break;
-      set_home(message);
+      home_auto=xstrdup(message);
+      Log::global->Format(9,"---- home set to %s\n",home_auto);
+      PropagateHomeAuto();
+      if(!home)
+	 set_home(home_auto);
+      LsCache::SetDirectory(this, home, true);
       break;
    case EXPECT_CWD:
       p=PopDirectory();
@@ -761,11 +764,10 @@ int Fish::HandleReplies()
 	 set_real_cwd(p);
 	 if(mode==CHANGE_DIR && RespQueueIsEmpty())
 	 {
-	    xfree(cwd);
-	    cwd=p;
-	    p=0;
+	    cwd.Set(p);
 	    eof=true;
 	 }
+	 LsCache::SetDirectory(this,p,true);
       }
       else
 	 SetError(NO_FILE,message);
@@ -1087,28 +1089,19 @@ int Fish::Done()
    return IN_PROGRESS;
 }
 
-void Fish::Suspend()
+void Fish::SuspendInternal()
 {
-   if(suspended)
-      return;
    if(recv_buf)
-   {
-      recv_buf_suspended=recv_buf->IsSuspended();
-      recv_buf->Suspend();
-   }
+      recv_buf->SuspendSlave();
    if(send_buf)
-      send_buf->Suspend();
-   super::Suspend();
+      send_buf->SuspendSlave();
 }
-void Fish::Resume()
+void Fish::ResumeInternal()
 {
-   if(!suspended)
-      return;
-   super::Resume();
-   if(recv_buf && !recv_buf_suspended)
-      recv_buf->Resume();
+   if(recv_buf)
+      recv_buf->ResumeSlave();
    if(send_buf)
-      send_buf->Resume();
+      send_buf->ResumeSlave();
 }
 
 const char *Fish::CurrentStatus()
@@ -1295,17 +1288,15 @@ const char *FishDirList::Status()
    return "";
 }
 
-void FishDirList::Suspend()
+void FishDirList::SuspendInternal()
 {
    if(ubuf)
-      ubuf->Suspend();
-   super::Suspend();
+      ubuf->SuspendSlave();
 }
-void FishDirList::Resume()
+void FishDirList::ResumeInternal()
 {
-   super::Resume();
    if(ubuf)
-      ubuf->Resume();
+      ubuf->ResumeSlave();
 }
 
 static FileSet *ls_to_FileSet(const char *b,int len)
