@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.192 2005/04/04 09:59:40 lav Exp $ */
+/* $Id: Http.cc,v 1.204 2005/12/31 07:01:36 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -70,7 +70,6 @@ void Http::Init()
    sock=-1;
    send_buf=0;
    recv_buf=0;
-   recv_buf_suspended=false;
    body_size=-1;
    bytes_received=0;
    line=0;
@@ -140,9 +139,10 @@ void Http::MoveConnectionHere(Http *o)
    sock=o->sock; o->sock=-1;
    rate_limit=o->rate_limit; o->rate_limit=0;
    last_method=o->last_method; o->last_method=0;
-   event_time=o->event_time;
+   BumpEventTime(o->event_time);
    state=CONNECTED;
    o->Disconnect();
+   ResumeInternal();
 }
 
 void Http::Disconnect()
@@ -334,7 +334,7 @@ void Http::SendMethod(const char *method,const char *efile)
       if(!xstrcmp(referer,"."))
       {
 	 referer=GetConnectURL(NO_USER+NO_PASSWORD);
-	 if(last_char(referer)!='/')
+	 if(last_char(referer)!='/' && !cwd.is_file)
 	    slash="/";
       }
       if(referer && referer[0])
@@ -436,35 +436,61 @@ bool Http::ModeSupported()
 
 void Http::DirFile(char *path_base,const char *ecwd,const char *efile)
 {
+   if(!strcmp(ecwd,"~") && !hftp)
+      ecwd="";
+   const char *sep=(last_char(ecwd)=='/'?"":"/");
+   if(efile[0]==0)
+      sep="";
+   const char *pre=(ecwd[0]=='/'?"":"/");
    if(efile[0]=='/')
       strcpy(path_base,efile);
    else if(efile[0]=='~')
       sprintf(path_base,"/%s",efile);
-   else if(cwd[0]==0 || ((cwd[0]=='/' || (!hftp && cwd[0]=='~')) && cwd[1]==0))
-      sprintf(path_base,"/%s",efile);
-   else if(cwd[0]=='~')
-      sprintf(path_base,"/%s/%s",ecwd,efile);
    else
-      sprintf(path_base,"%s/%s",ecwd,efile);
+      sprintf(path_base,"%s%s%s%s",pre,ecwd,sep,efile);
 
-   if(path_base[1]=='~' && path_base[2]=='/')
+   if(path_base[1]=='~' && path_base[2]==0)
+      path_base[1]=0;
+   else if(path_base[1]=='~' && path_base[2]=='/')
       memmove(path_base,path_base+2,strlen(path_base+2)+1);
-   else if(hftp && path_base[1]!='~')
-   {
-      // root directory in ftp urls needs special encoding. (/%2Fpath)
-      memmove(path_base+4,path_base+1,strlen(path_base+1)+1);
-      memcpy(path_base+1,"%2F",3);
-   }
 }
 
 void Http::SendRequest(const char *connection,const char *f)
 {
    char *efile=string_alloca(strlen(f)*3+1);
    url::encode_string(f,efile,URL_PATH_UNSAFE);
-   char *ecwd=string_alloca(strlen(cwd)*3+1);
-   url::encode_string(cwd,ecwd,URL_PATH_UNSAFE);
+   if(mode==CHANGE_DIR && new_cwd && new_cwd->url)
+      efile=new_cwd->url+url::path_index(new_cwd->url);
+   char *ecwd;
    int efile_len;
 
+   if(cwd.url)
+   {
+      ecwd=cwd.url+url::path_index(cwd.url);
+      ecwd=alloca_strdup(ecwd);
+   }
+   else
+   {
+      ecwd=string_alloca(strlen(cwd)*3+3+1);
+      url::encode_string(cwd,ecwd,URL_PATH_UNSAFE);
+      if(hftp && ecwd[0]=='/' && ecwd[1]!='~')
+      {
+	 // root directory in ftp urls needs special encoding. (/%2Fpath)
+	 memmove(ecwd+4,ecwd+1,strlen(ecwd+1)+1);
+	 memcpy(ecwd+1,"%2F",3);
+      }
+   }
+   bool add_slash=true;
+   if(cwd.is_file)
+   {
+      if(efile[0])
+	 dirname_modify(ecwd+(!strncmp(ecwd,"/~",2)));
+      else
+	 add_slash=false;
+   }
+   if(mode==CHANGE_DIR && new_cwd)
+      add_slash=!new_cwd->is_file;
+
    char *pfile=string_alloca(4+3+xstrlen(user)*6+3+xstrlen(pass)*3+1+
 			      strlen(hostname)*3+1+xstrlen(portname)*3+1+
 			      strlen(ecwd)+1+strlen(efile)+1+6+1);
@@ -585,7 +611,7 @@ void Http::SendRequest(const char *connection,const char *f)
    case LONG_LIST:
    case MP_LIST:
    case MAKE_DIR:
-      if(efile_len<1 || efile[efile_len-1]!='/')
+      if(last_char(efile)!='/' && add_slash)
 	 strcat(efile,"/");
       if(mode==CHANGE_DIR)
 	 SendMethod("HEAD",efile);
@@ -772,9 +798,10 @@ void Http::HandleHeaderLine(const char *name,const char *value)
    {
       keep_alive=true;
       const char *m=strstr(value,"max=");
-      if(m)
-	 sscanf(m+4,"%d",&keep_alive_max);
-      else
+      if(m) {
+	 if(sscanf(m+4,"%d",&keep_alive_max)!=1)
+	    keep_alive=false;
+      } else
 	 keep_alive_max=100;
       return;
    }
@@ -874,7 +901,7 @@ void Http::GetBetterConnection(int level)
       {
 	 if(level<2)
 	    continue;
-	 if(!connection_takeover || (o->priority>=priority && !o->suspended))
+	 if(!connection_takeover || (o->priority>=priority && !o->IsSuspended()))
 	    continue;
 	 o->Disconnect();
 	 return;
@@ -914,8 +941,8 @@ int Http::Do()
       TimeoutS(idle_start+idle-time_t(now));
    }
 
-   if(home==0)
-      home=xstrdup(default_cwd);
+   if(home.path==0)
+      home.Set(default_cwd);
    ExpandTildeInCWD();
 
    if(Error())
@@ -1012,7 +1039,7 @@ int Http::Do()
 	    return MOVED;
       }
 
-      if(!ReconnectAllowed())
+      if(!resolver && mode!=CONNECT_VERIFY && !ReconnectAllowed())
 	 return m;
 
       if(https)
@@ -1025,6 +1052,9 @@ int Http::Do()
       if(mode==CONNECT_VERIFY)
 	 return m;
 
+      if(!ReconnectAllowed())
+	 return m;
+
       if(!NextTry())
       	 return MOVED;
 
@@ -1091,7 +1121,7 @@ int Http::Do()
 
       m=MOVED;
       state=CONNECTED;
-#ifdef USE_SSL
+#if USE_SSL
       if(proxy?!strncmp(proxy,"https://",8):https)
       {
 	 MakeSSLBuffers();
@@ -1103,7 +1133,7 @@ int Http::Do()
 	    new FDStream(sock,"<output-socket>"),IOBuffer::PUT);
 	 recv_buf=new IOBufferFDStream(
 	    new FDStream(sock,"<input-socket>"),IOBuffer::GET);
-#ifdef USE_SSL
+#if USE_SSL
 	 if(proxy && https)
 	 {
 	    // have to setup a tunnel.
@@ -1213,7 +1243,7 @@ int Http::Do()
 	       {
 		  if(H_20X(status_code))
 		  {
-#ifdef USE_SSL
+#if USE_SSL
 		     if(https)
 			MakeSSLBuffers();
 #endif
@@ -1466,7 +1496,7 @@ int Http::Do()
 	    if(closure && closure[0])
 	       sprintf(err,"%s (%s)",status+status_consumed,closure);
 	    else
-	       sprintf(err,"%s (%s%s)",status+status_consumed,cwd,
+	       sprintf(err,"%s (%s%s)",status+status_consumed,cwd.path,
 				       (last_char(cwd)=='/')?"":"/");
 	 }
 	 state=RECEIVING_BODY;
@@ -1475,8 +1505,7 @@ int Http::Do()
       }
       if(mode==CHANGE_DIR)
       {
-	 xfree(cwd);
-	 cwd=xstrdup(file);
+	 cwd.Set(new_cwd);
 	 state=DONE;
 	 return MOVED;
       }
@@ -1557,33 +1586,24 @@ void  Http::ClassInit()
    // register the class
    Register("http",Http::New);
    Register("hftp",HFtp::New);
-#ifdef USE_SSL
+#if USE_SSL
    Register("https",Https::New);
 #endif
 }
 
-void Http::Suspend()
+void Http::SuspendInternal()
 {
-   if(suspended)
-      return;
    if(recv_buf)
-   {
-      recv_buf_suspended=recv_buf->IsSuspended();
-      recv_buf->Suspend();
-   }
+      recv_buf->SuspendSlave();
    if(send_buf)
-      send_buf->Suspend();
-   super::Suspend();
+      send_buf->SuspendSlave();
 }
-void Http::Resume()
+void Http::ResumeInternal()
 {
-   if(!suspended)
-      return;
-   super::Resume();
-   if(recv_buf && !recv_buf_suspended)
-      recv_buf->Resume();
+   if(recv_buf)
+      recv_buf->ResumeSlave();
    if(send_buf)
-      send_buf->Resume();
+      send_buf->ResumeSlave();
 }
 
 int Http::Read(void *buf,int size)
@@ -1904,7 +1924,7 @@ bool Http::SameLocationAs(FileAccess *fa)
    if(!SameSiteAs(fa))
       return false;
    Http *o=(Http*)fa;
-   if(xstrcmp(cwd,o->cwd))
+   if(cwd!=o->cwd)
       return false;
    return true;
 }
@@ -2127,7 +2147,7 @@ void Http::SetCookie(const char *value_const)
    xfree(c);
 }
 
-#ifdef USE_SSL
+#if USE_SSL
 #undef super
 #define super Http
 Https::Https()
@@ -2151,10 +2171,9 @@ void Http::MakeSSLBuffers()
    Delete(send_buf);
    Delete(recv_buf);
 
-   SSL *ssl=lftp_ssl_new(sock,hostname);
-   IOBufferSSL *send_buf_ssl=new IOBufferSSL(ssl,IOBuffer::PUT,hostname);
-   IOBufferSSL *recv_buf_ssl=new IOBufferSSL(ssl,IOBuffer::GET,hostname);
-   send_buf_ssl->DoConnect();
+   lftp_ssl *ssl=new lftp_ssl(sock,lftp_ssl::CLIENT,hostname);
+   IOBufferSSL *send_buf_ssl=new IOBufferSSL(ssl,IOBuffer::PUT);
+   IOBufferSSL *recv_buf_ssl=new IOBufferSSL(ssl,IOBuffer::GET);
    recv_buf_ssl->CloseLater();
    send_buf=send_buf_ssl;
    recv_buf=recv_buf_ssl;
@@ -2182,10 +2201,8 @@ void HFtp::Login(const char *u,const char *p)
    super::Login(u,p);
    if(u)
    {
-      xfree(home);
-      home=xstrdup("~");
-      xfree(cwd);
-      cwd=xstrdup(home);
+      home.Set("~");
+      cwd.Set(home,false,0);
    }
 }
 void HFtp::Reconfig(const char *name)
