@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: HttpDir.cc,v 1.85 2005/04/04 09:59:41 lav Exp $ */
+/* $Id: HttpDir.cc,v 1.91 2005/12/31 07:01:36 lav Exp $ */
 
 #include <config.h>
 #include "ascii_ctype.h"
@@ -688,6 +688,9 @@ static int parse_html(const char *buf,int len,bool eof,Buffer *list,
    if(more-less<3)
       return tag_len;   // too small
 
+   if(less[1]=='/' || less[1]=='!')
+      return tag_len;
+
    int max_link_len=more-less+1+2;
    if(base_href && *base_href)
       max_link_len+=strlen(*base_href)+1;
@@ -745,25 +748,29 @@ static int parse_html(const char *buf,int len,bool eof,Buffer *list,
 
    decode_amps(link_target);  // decode all &amp; and similar
 
+   if(hftp)
+   {
+      // workaround proxy bugs.
+      char *t=strstr(link_target,";type=");
+      if(t && t[6] && t[7]=='/' && t[8]==0)
+	 *t=0;
+      char *p=link_target+url::path_index(link_target);
+      if(p[0]=='/' && p[1]=='/')
+      {
+	 memmove(p+4,p+2,strlen(p+2)+1);
+	 memcpy(p+1,"%2F",3);
+      }
+   }
+
+   Log::global->Format(10,"Found tag %s, link_target=%s\n",tag_scan->tag,link_target);
+
    if(!strcasecmp(tag_scan->tag,"base"))
    {
       if(base_href)
       {
 	 xfree(*base_href);
 	 *base_href=xstrdup(link_target,+2);
-	 if(hftp)
-	 {
-	    // workaround apache proxy bugs.
-	    char *t=strstr(*base_href,";type=");
-	    if(t && t[6] && t[7]=='/' && t[8]==0)
-	       *t=0;
-	    char *p=*base_href+url::path_index(*base_href);
-	    if(p[0]=='/' && p[1]=='/')
-      	    {
-	       memmove(p+4,p+2,strlen(p+2)+1);
-	       memcpy(p+1,"%2F",3);
-	    }
-	 }
+	 Log::global->Format(10,"Found base href=%s\n",*base_href);
       }
       return tag_len;
    }
@@ -875,12 +882,18 @@ parse_url_again:
    if(info.is_directory && link_len>1)
       link_target[--link_len]=0;
 
+   FileAccess::Path::Optimize(link_target+(link_target[0]=='/' && link_target[1]=='~'));
+   link_len=strlen(link_target);
+
    if(prefix)
    {
-      const char *p_path=prefix->path;
-      if(p_path==0)
-	 p_path="~";
+      const char *p_path_c=prefix->path;
+      if(p_path_c==0)
+	 p_path_c="~";
+      char *p_path=alloca_strdup(p_path_c);
       int p_len=strlen(p_path);
+      if(p_len>1 && p_path[p_len-1]=='/')
+	 p_path[--p_len]=0;
       if(p_len==1 && p_path[0]=='/' && link_target[0]=='/')
       {
 	 if(link_len>1)
@@ -934,6 +947,13 @@ parse_url_again:
       type[0]=';';
    }
 
+   if(link_len==0)
+   {
+      strcpy(link_target,".");
+      link_len=1;
+      info.is_directory=true;
+   }
+
    bool show_in_list=true;
    if(icon && (link_target[0]=='/' || link_target[0]=='~'))
       show_in_list=false;  // makes apache listings look better.
@@ -1073,7 +1093,12 @@ parse_url_again:
       if(info.size==-1)
       {
 	 if(strspn(info.size_str,"0123456789")==strlen(info.size_str))
-	    info.size=atoll(info.size_str);
+	 {
+	    long long size_ll=0;
+	    if(sscanf(info.size_str,"%lld",&size_ll)!=1)
+	       size_ll=0;
+	    info.size=size_ll;
+	 }
       }
       if(info.perms[0]==0)
       {
@@ -1257,6 +1282,11 @@ int HttpDirList::Do()
       }
       else
       {
+	 if(mode==FA::MP_LIST && !*curr && session->GetCwd().is_file)
+	 {
+	    mode=FA::LONG_LIST;
+	    goto retry;
+	 }
 	 session->Open(curr,mode);
 	 session->UseCache(use_cache);
 	 ubuf=new IOBufferFileAccess(session);
@@ -1374,17 +1404,15 @@ const char *HttpDirList::Status()
    return "";
 }
 
-void HttpDirList::Suspend()
+void HttpDirList::SuspendInternal()
 {
    if(ubuf)
-      ubuf->Suspend();
-   super::Suspend();
+      ubuf->SuspendSlave();
 }
-void HttpDirList::Resume()
+void HttpDirList::ResumeInternal()
 {
-   super::Resume();
    if(ubuf)
-      ubuf->Resume();
+      ubuf->ResumeSlave();
 }
 
 
