@@ -18,14 +18,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Job.cc,v 1.25 2002/03/15 14:27:26 lav Exp $ */
+/* $Id: Job.cc,v 1.33 2005/10/18 07:54:52 lav Exp $ */
 
 #include <config.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <assert.h>
 #include "Job.h"
-#include "xalloca.h"
 
 Job *Job::chain;
 
@@ -53,29 +52,29 @@ void  Job::AllocJobno()
 
 Job::~Job()
 {
-   // first, reparent or kill children (hm, that's sadistic)
+   xfree(waiting);
+   waiting=0;
+   waiting_num=0;
+   // reparent or kill children (hm, that's sadistic)
    {
       for(Job *scan=chain; scan; )
       {
 	 if(scan->parent==this)
 	 {
 	    if(scan->jobno!=-1 && this->parent)
-	    {
 	       scan->parent=this->parent;
-	       scan=scan->next;
-	    }
-	    else if(!scan->deleting)
-	    {
-	       Delete(scan);
-	       scan=chain;
-	    }
 	    else
-	       scan=scan->next;
+	    {
+	       scan->parent=0;
+	       scan->deleting=true;
+	    }
 	 }
-	 else
-	    scan=scan->next;
+	 scan=scan->next;
       }
    }
+   // if parent waits for the job, make it stop
+   if(parent)
+      parent->RemoveWaiting(this);
    // now, delete the job from the list
    {
       for(Job **scan=&chain; *scan; scan=&(*scan)->next)
@@ -186,24 +185,27 @@ public:
    int	 ExitCode() { return 255; }
 };
 
+void Job::Kill(Job *j)
+{
+   if(j->parent && j->parent->WaitsFor(j))
+   {
+      // someone waits for termination of this job, so
+      // we have to simulate normal death...
+      Job *r=new KilledJob();
+      r->parent=j->parent;
+      r->cmdline=j->cmdline;
+      j->cmdline=0;
+      j->parent->ReplaceWaiting(j,r);
+   }
+   assert(FindWhoWaitsFor(j)==0);
+   Delete(j);
+}
+
 void Job::Kill(int n)
 {
    Job *j=FindJob(n);
    if(j)
-   {
-      if(j->parent && j->parent->WaitsFor(j))
-      {
-	 // someone waits for termination of this job, so
-	 // we have to simulate normal death...
-	 Job *r=new KilledJob();
-	 r->parent=j->parent;
-	 r->cmdline=j->cmdline;
-	 j->cmdline=0;
-	 j->parent->ReplaceWaiting(j,r);
-      }
-      assert(FindWhoWaitsFor(j)==0);
-      Delete(j);
-   }
+      Kill(j);
 }
 
 void Job::KillAll()
@@ -213,7 +215,7 @@ void Job::KillAll()
       Job *tmp=scan;
       scan=scan->next;
       if(tmp->jobno>=0)
-	 Delete(tmp);
+	 Job::Kill(tmp);
    }
 }
 
@@ -349,17 +351,13 @@ void  Job::ListDoneJobs()
 
 void  Job::BuryDoneJobs()
 {
-   for(Job *scan=chain; scan; )
+   for(Job *scan=chain; scan; scan=scan->next)
    {
       if((scan->parent==this || scan->parent==0) && scan->jobno>=0
 		  && scan->Done())
-      {
-	 Delete(scan);
-      	 scan=chain;
-      }
-      else
-	 scan=scan->next;
+	 scan->deleting=true;
    }
+   CollectGarbage();
 }
 
 void Job::fprintf(FILE *file,const char *fmt,...)
@@ -406,13 +404,13 @@ SessionJob::~SessionJob()
    session=0;
 }
 
-void SessionJob::PrintStatus(int v)
+void SessionJob::PrintStatus(int v,const char *prefix)
 {
    if(v<2 || !session)
       return;
    const char *url=session->GetConnectURL();
    if(url && *url)
-      printf("\t%s\n",url);
+      printf("%s%s\n",prefix,url);
 }
 
 void SessionJob::Fg()
@@ -473,6 +471,7 @@ void Job::Bg()
 }
 void Job::Fg()
 {
+   Resume();
    if(fg)
       return;
    fg=true;
