@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: LsCache.h,v 1.14 2001/12/13 14:25:55 lav Exp $ */
+/* $Id: LsCache.h,v 1.17 2005/04/04 09:59:42 lav Exp $ */
 
 #ifndef LSCACHE_H
 #define LSCACHE_H
@@ -30,9 +30,13 @@ class Buffer;
 
 class LsCache
 {
-   time_t   timestamp;
+   time_t timestamp;
+   int	 err_code;
+   time_t ExpireTime();
+   bool	 Expired();
    char	 *data;
    int	 data_len;
+   FileSet *afset;    // associated file set
 
    char	 *arg;
    FileAccess *loc;
@@ -43,12 +47,7 @@ class LsCache
    static bool use;
    static long sizelimit;
    static TimeInterval ttl;
-   /* single cached fileset */
-   static FileSet *fset;
-   static FileAccess *fset_loc;
-   static int fset_m;
-   static char *fset_a;
-   static void free_fset();
+   static TimeInterval ttl_neg;
 
    class ExpireHelper;
    friend class LsCache::ExpireHelper;
@@ -58,6 +57,7 @@ class LsCache
       LsCache *expiring;
       int Do();
       ExpireHelper() { expiring=0; }
+      void Reconfig(const char *name);
    };
    static ExpireHelper expire_helper;
 
@@ -67,9 +67,9 @@ protected:
    ~LsCache();
 
 public:
-   static void Add(FileAccess *p_loc,const char *a,int m,const char *d,int l);
-   static void Add(FileAccess *p_loc,const char *a,int m,const Buffer *ubuf);
-   static int Find(FileAccess *p_loc,const char *a,int m,const char **d, int *l);
+   static void Add(FileAccess *p_loc,const char *a,int m,int err,const char *d,int l,const FileSet *f=0);
+   static void Add(FileAccess *p_loc,const char *a,int m,int err,const Buffer *ubuf,const FileSet *f=0);
+   static int Find(FileAccess *p_loc,const char *a,int m,int *err,const char **d, int *l,FileSet **f=0);
    static FileSet *FindFileSet(FileAccess *p_loc,const char *a,int m);
 
    static int IsDirectory(FileAccess *p_loc,const char *dir);
@@ -98,6 +98,7 @@ public:
    static void SetSizeLimit(long l) { sizelimit=l; }
    static long SizeLimit() { return sizelimit; }
    static void SetExpire(const TimeInterval &t) { ttl=t; }
+   static void SetExpireNegative(const TimeInterval &t) { ttl_neg=t; }
 };
 
 #endif//LSCACHE_H
