@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.119 2005/04/07 15:30:21 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.130 2005/12/15 08:49:47 lav Exp $ */
 
 #include <config.h>
 
@@ -27,6 +27,7 @@
 #include <errno.h>
 #include <unistd.h>
 #include <assert.h>
+#include <mbswidth.h>
 #include "MirrorJob.h"
 #include "CmdExec.h"
 #include "rmJob.h"
@@ -104,17 +105,22 @@ final:
       printf(plural("%sModified: %d file$|s$, %d symlink$|s$\n",
 		     stats.mod_files,stats.mod_symlinks),
 	 tab,stats.mod_files,stats.mod_symlinks);
+   if(stats.bytes)
+      printf("%s%s\n",tab,CopyJob::FormatBytesTimeRate(stats.bytes,stats.time));
    if(stats.del_dirs || stats.del_files || stats.del_symlinks)
       printf(plural(flags&DELETE ?
 	       "%sRemoved: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n"
 	      :"%sTo be removed: %d director$y|ies$, %d file$|s$, %d symlink$|s$\n",
 	      stats.del_dirs,stats.del_files,stats.del_symlinks),
 	 tab,stats.del_dirs,stats.del_files,stats.del_symlinks);
-   return;
+   if(stats.error_count)
+      printf(plural("%s%d error$|s$ detected\n",stats.error_count),
+	       tab,stats.error_count);
 }
 
 void  MirrorJob::ShowRunStatus(StatusLine *s)
 {
+   int w=s->GetWidthDelayed();
    switch(state)
    {
    case(INITIAL_STATE):
@@ -145,19 +151,27 @@ void  MirrorJob::ShowRunStatus(StatusLine *s)
    case(GETTING_LIST_INFO):
       if(target_list_info && (!source_list_info || now%4>=2))
       {
+	 const char *status=target_list_info->Status();
+	 int status_w=mbswidth(status, MBSW_ACCEPT_INVALID|MBSW_ACCEPT_UNPRINTABLE);
+	 int dw=w-status_w;
+	 if(dw<20)
+	    dw=20;
 	 if(target_relative_dir)
-	    s->Show("%s: %s",squeeze_file_name(target_relative_dir,20),
-	       target_list_info->Status());
+	    s->Show("%s: %s",squeeze_file_name(target_relative_dir,dw),status);
 	 else
-	    s->Show("%s",target_list_info->Status());
+	    s->Show("%s",status);
       }
       else if(source_list_info)
       {
+	 const char *status=source_list_info->Status();
+	 int status_w=mbswidth(status, MBSW_ACCEPT_INVALID|MBSW_ACCEPT_UNPRINTABLE);
+	 int dw=w-status_w;
+	 if(dw<20)
+	    dw=20;
 	 if(source_relative_dir)
-	    s->Show("%s: %s",squeeze_file_name(source_relative_dir,20),
-	       source_list_info->Status());
+	    s->Show("%s: %s",squeeze_file_name(source_relative_dir,dw),status);
 	 else
-	    s->Show("%s",source_list_info->Status());
+	    s->Show("%s",status);
       }
       break;
    }
@@ -365,6 +379,8 @@ void  MirrorJob::HandleFile(FileInfo *file)
 
 	 mj->verbose_report=verbose_report;
 	 mj->newer_than=newer_than;
+	 mj->older_than=older_than;
+	 mj->size_range=size_range;
 	 mj->parallel=parallel;
 	 mj->pget_n=pget_n;
 	 mj->pget_minchunk=pget_minchunk;
@@ -377,6 +393,8 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 mj->script_name=xstrdup(script_name);
 	 mj->script_only=script_only;
 
+	 mj->max_error_count=max_error_count;
+
 	 if(verbose_report>=3)
 	    Report(_("Mirroring directory `%s'"),mj->target_relative_dir);
 
@@ -387,6 +405,9 @@ void  MirrorJob::HandleFile(FileInfo *file)
 	 if(flags&NO_SYMLINKS)
 	    goto skip;
 
+	 if(!file->symlink)
+	    goto skip;
+
 	 if(!target_is_local)
 	 {
 	    // can't create symlink remotely (FIXME?)
@@ -467,6 +488,10 @@ void  MirrorJob::InitSets(FileSet *source,FileSet *dest)
 
    if(newer_than!=(time_t)-1)
       to_transfer->SubtractOlderThan(newer_than);
+   if(older_than!=(time_t)-1)
+      to_transfer->SubtractNewerThan(older_than);
+   if(size_range)
+      to_transfer->SubtractSizeOutside(size_range);
 
    if(flags&NO_RECURSION)
       to_transfer->SubtractDirs();
@@ -562,7 +587,12 @@ void MirrorJob::HandleListInfoCreation(FileAccess * &session,ListInfo * &list_in
       return;
    }
    list_info->UseCache(use_cache);
-   list_info->Need(FileInfo::ALL_INFO);
+   int need=FileInfo::ALL_INFO;
+   if(flags&IGNORE_TIME)
+      need&=~FileInfo::DATE;
+   if(flags&IGNORE_SIZE)
+      need&=~FileInfo::SIZE;
+   list_info->Need(need);
    if(flags&RETR_SYMLINKS)
       list_info->FollowSymlinks();
 
@@ -730,8 +760,10 @@ int   MirrorJob::Do()
       m=MOVED;
       /*fallthrough*/
    case(WAITING_FOR_TRANSFER):
-      while(j=FindDoneAwaitedJob())
+      while((j=FindDoneAwaitedJob())!=0)
       {
+	 stats.bytes+=j->GetBytesCount();
+	 stats.time +=j->GetTimeSpent();
 	 if(j->ExitCode()!=0)
 	    stats.error_count++;
 	 RemoveWaiting(j);
@@ -739,6 +771,8 @@ int   MirrorJob::Do()
 	 transfer_count--;
 	 m=MOVED;
       }
+      if(max_error_count>0 && stats.error_count>=max_error_count)
+	 goto pre_FINISHING;
       while(transfer_count<parallel && state==WAITING_FOR_TRANSFER)
       {
 	 file=to_transfer->curr();
@@ -763,13 +797,15 @@ int   MirrorJob::Do()
    case(TARGET_REMOVE_OLD):
    case(TARGET_REMOVE_OLD_FIRST):
    TARGET_REMOVE_OLD_FIRST_label:
-      while(j=FindDoneAwaitedJob())
+      while((j=FindDoneAwaitedJob())!=0)
       {
 	 RemoveWaiting(j);
 	 Delete(j);
 	 transfer_count--;
 	 m=MOVED;
       }
+      if(max_error_count>0 && stats.error_count>=max_error_count)
+	 goto pre_FINISHING;
       while(transfer_count<parallel && (state==TARGET_REMOVE_OLD || state==TARGET_REMOVE_OLD_FIRST))
       {
 	 file=0;
@@ -808,7 +844,12 @@ int   MirrorJob::Do()
 	 {
 	    ArgV args("rm");
 	    if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
-	       args.Append("-r");
+	    {
+	       if(flags&NO_RECURSION)
+		  args.setarg(0,"rmdir");
+	       else
+		  args.Append("-r");
+	    }
 	    args.Append(target_session->GetFileURL(file->name));
 	    char *cmd=args.CombineQuoted();
 	    fprintf(script,"%s\n",cmd);
@@ -825,7 +866,15 @@ int   MirrorJob::Do()
 	    AddWaiting(j);
 	    transfer_count++;
 	    if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
-	       j->Recurse();
+	    {
+	       if(flags&NO_RECURSION)
+	       {
+		  args->setarg(0,"rmdir");
+		  j->Rmdir();
+	       }
+	       else
+		  j->Recurse();
+	    }
 	 }
 	 if(file->defined&file->TYPE && file->filetype==file->DIRECTORY)
 	 {
@@ -849,13 +898,15 @@ int   MirrorJob::Do()
       m=MOVED;
       /*fallthrough*/
    case(TARGET_CHMOD):
-      while(j=FindDoneAwaitedJob())
+      while((j=FindDoneAwaitedJob())!=0)
       {
 	 RemoveWaiting(j);
 	 Delete(j);
 	 transfer_count--;
 	 m=MOVED;
       }
+      if(max_error_count>0 && stats.error_count>=max_error_count)
+	 goto pre_FINISHING;
       while(transfer_count<parallel && state==TARGET_CHMOD)
       {
 	 file=to_transfer->curr();
@@ -923,7 +974,7 @@ int   MirrorJob::Do()
       m=MOVED;
       /*fallthrough*/
    case(FINISHING):
-      while(j=FindDoneAwaitedJob())
+      while((j=FindDoneAwaitedJob())!=0)
       {
 	 RemoveWaiting(j);
 	 Delete(j);
@@ -938,7 +989,7 @@ int   MirrorJob::Do()
 	 parent_mirror->stats.Add(stats);
       else
       {
-	 if((flags&LOOP) && stats.HaveSomethingDone() && !stats.error_count)
+	 if((flags&LOOP) && stats.HaveSomethingDone(flags) && !stats.error_count)
 	 {
 	    set_state(DONE);
 	    PrintStatus(0,"");
@@ -989,12 +1040,15 @@ MirrorJob::MirrorJob(MirrorJob *parent,
    target_list_info=0;
 
    flags=0;
+   max_error_count=0;
 
    exclude=0;
 
    set_state(INITIAL_STATE);
 
    newer_than=(time_t)-1;
+   older_than=(time_t)-1;
+   size_range=0;
 
    script_name=0;
    script=0;
@@ -1042,7 +1096,11 @@ MirrorJob::~MirrorJob()
    SessionPool::Reuse(source_session);
    SessionPool::Reuse(target_session);
    if(!parent_mirror)
+   {
+      // common pointers, delete only on top level.
       delete exclude;
+      delete size_range;
+   }
    if(script && script_needs_closing)
       fclose(script);
 }
@@ -1058,7 +1116,7 @@ void MirrorJob::va_Report(const char *fmt,va_list v)
    if(verbose_report)
    {
       pid_t p=tcgetpgrp(fileno(stdout));
-      if(p!=-1 && p!=getpgrp())
+      if(p>0 && p!=getpgrp())
 	 return;
 
       vfprintf(stdout,fmt,v);
@@ -1097,6 +1155,23 @@ void MirrorJob::SetNewerThan(const char *f)
    }
    newer_than=st.st_mtime;
 }
+void MirrorJob::SetOlderThan(const char *f)
+{
+   time_t t=now;
+   t=get_date(f,&t);
+   if(t>0)
+   {
+      older_than=t;
+      return;
+   }
+   struct stat st;
+   if(stat(f,&st)==-1)
+   {
+      perror(f);
+      return;
+   }
+   older_than=st.st_mtime;
+}
 
 mode_t MirrorJob::get_mode_mask()
 {
@@ -1159,9 +1234,10 @@ void MirrorJob::Statistics::Add(const Statistics &s)
    bytes       +=s.bytes;
    time	       +=s.time;
 }
-bool MirrorJob::Statistics::HaveSomethingDone()
+bool MirrorJob::Statistics::HaveSomethingDone(int flags)
 {
-   return new_files|mod_files|del_files|new_symlinks|mod_symlinks|del_symlinks|del_dirs;
+   bool del=(flags&MirrorJob::DELETE);
+   return new_files|mod_files|(del_files*del)|new_symlinks|mod_symlinks|(del_symlinks*del)|(del_dirs*del);
 }
 
 char *MirrorJob::SetScriptFile(const char *n)
@@ -1203,6 +1279,8 @@ CMD(mirror)
       {"reverse",no_argument,0,'R'},
       {"verbose",optional_argument,0,'v'},
       {"newer-than",required_argument,0,'N'},
+      {"older-than",required_argument,0,256+'O'},
+      {"size-range",required_argument,0,512+'S'},
       {"dereference",no_argument,0,'L'},
       {"use-cache",no_argument,0,256+'C'},
       {"Remove-source-files",no_argument,0,256+'R'},
@@ -1218,11 +1296,13 @@ CMD(mirror)
       {"use-pget-n",optional_argument,0,256+'p'},
       {"no-symlinks",no_argument,0,256+'y'},
       {"loop",no_argument,0,256+'l'},
+      {"max-errors",required_argument,0,256+'E'},
       {0}
    };
 
    int opt;
    int flags=0;
+   int max_error_count=0;
 
    bool use_cache=false;
 
@@ -1231,6 +1311,8 @@ CMD(mirror)
 
    int	 verbose=0;
    const char *newer_than=0;
+   const char *older_than=0;
+   Range *size_range=0;
    bool  remove_source_files=false;
    int	 parallel=ResMgr::Query("mirror:parallel-transfer-count",0);
    int	 use_pget=ResMgr::Query("mirror:use-pget-n",0);
@@ -1331,6 +1413,18 @@ CMD(mirror)
       case('N'):
 	 newer_than=optarg;
 	 break;
+      case(256+'O'):
+	 older_than=optarg;
+	 break;
+      case(512+'S'):
+	 size_range=new Range(optarg);
+	 if(size_range->Error())
+	 {
+	    eprintf("%s: --size-range \"%s\": %s\n",
+	       args->a0(),optarg,size_range->ErrorText());
+	    goto no_job;
+	 }
+	 break;
       case(256+'u'):
 	 flags|=MirrorJob::NO_UMASK;
 	 break;
@@ -1377,10 +1471,14 @@ CMD(mirror)
       case(256+'l'):
 	 flags|=MirrorJob::LOOP;
 	 break;
+      case(256+'E'):
+	 max_error_count=atoi(optarg);
+	 break;
       case('?'):
 	 eprintf(_("Try `help %s' for more information.\n"),args->a0());
       no_job:
 	 delete exclude;
+	 delete size_range;
 	 if(source_session)
 	    SMTask::Delete(source_session);
 	 if(target_session)
@@ -1433,7 +1531,7 @@ CMD(mirror)
 	    }
 	    target_dir=alloca_strdup(target_url.path);
 	 }
-	 if(last_char(target_dir)=='/' && basename_ptr(target_dir)[0]!='/')
+	 if(last_char(arg)=='/' && basename_ptr(arg)[0]!='/')
 	 {
 	    // user wants source dir name appended.
 	    const char *base=basename_ptr(source_dir);
@@ -1477,6 +1575,10 @@ CMD(mirror)
 
    if(newer_than)
       j->SetNewerThan(newer_than);
+   if(older_than)
+      j->SetOlderThan(older_than);
+   if(size_range)
+      j->SetSizeRange(size_range);
    j->UseCache(use_cache);
    if(remove_source_files)
       j->RemoveSourceFiles();
@@ -1506,6 +1608,7 @@ CMD(mirror)
       if(!script_file)
 	 j->SetScriptFile("-");
    }
+   j->SetMaxErrorCount(max_error_count);
 
    return j;
 
