@@ -18,13 +18,31 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.cc,v 1.56 2002/07/18 09:13:48 lav Exp $ */
+/* $Id: NetAccess.cc,v 1.79 2005/06/14 13:35:32 lav Exp $ */
 
 #include <config.h>
 
 #include <errno.h>
 #include <assert.h>
 #include <math.h>
+#include <sys/types.h>
+#include <netinet/in.h>
+#ifdef HAVE_NETINET_IN_SYSTM_H
+# include <netinet/in_systm.h>
+#endif
+#ifdef HAVE_NETINET_IP_H
+# include <netinet/ip.h>
+#endif
+#ifdef HAVE_NETINET_TCP_H
+# include <netinet/tcp.h>
+#endif
+#include <arpa/inet.h>
+#ifdef HAVE_SYS_IOCTL_H
+# include <sys/ioctl.h>
+#endif
+#ifdef HAVE_TERMIOS_H
+# include <termios.h>
+#endif
 #include "NetAccess.h"
 #include "log.h"
 #include "url.h"
@@ -41,6 +59,7 @@ void NetAccess::Init()
    idle_start=now;
    max_retries=0;
    max_persist_retries=0;
+   retries=0;
    persist_retries=0;
    socket_buffer=0;
    socket_maxseg=0;
@@ -54,7 +73,7 @@ void NetAccess::Init()
    reconnect_interval_max=300;
    timeout=600;		   // 10 minutes with no events = reconnect
 
-   proxy=0;
+   proxy=proxy_proto=0;
    proxy_port=0;
    proxy_user=proxy_pass=0;
 
@@ -63,6 +82,8 @@ void NetAccess::Init()
    connection_limit=0;	// no limit.
    connection_takeover=false;
 
+   home_auto=xstrdup(FindHomeAuto());
+
    Reconfig(0);
 }
 
@@ -73,6 +94,8 @@ NetAccess::NetAccess()
 NetAccess::NetAccess(const NetAccess *o) : super(o)
 {
    Init();
+   xfree(home_auto);
+   home_auto=xstrdup(o->home_auto);
 }
 NetAccess::~NetAccess()
 {
@@ -81,10 +104,12 @@ NetAccess::~NetAccess()
       delete rate_limit;
    ClearPeer();
 
-   xfree(proxy); proxy=0;
-   xfree(proxy_port); proxy_port=0;
-   xfree(proxy_user); proxy_user=0;
-   xfree(proxy_pass); proxy_pass=0;
+   xfree(proxy);
+   xfree(proxy_port);
+   xfree(proxy_user);
+   xfree(proxy_pass);
+   xfree(proxy_proto);
+   xfree(home_auto);
 }
 
 void NetAccess::Reconfig(const char *name)
@@ -93,7 +118,7 @@ void NetAccess::Reconfig(const char *name)
 
    const char *c=hostname;
 
-   timeout = ResMgr::Query("net:timeout",c);
+   timeout = TimeInterval(ResMgr::Query("net:timeout",c)).Seconds();
    reconnect_interval = ResMgr::Query("net:reconnect-interval-base",c);
    reconnect_interval_multiplier = ResMgr::Query("net:reconnect-interval-multiplier",c);
    if(reconnect_interval_multiplier<1)
@@ -118,6 +143,22 @@ void NetAccess::KeepAlive(int sock)
 {
    setsockopt(sock,SOL_SOCKET,SO_KEEPALIVE,(char*)&one,sizeof(one));
 }
+void NetAccess::MinimizeLatency(int sock)
+{
+   int tos;
+#ifdef IP_TOS
+   tos = IPTOS_LOWDELAY;
+   setsockopt(sock, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int));
+#endif
+}
+void NetAccess::MaximizeThroughput(int sock)
+{
+   int tos;
+#ifdef IP_TOS
+   tos = IPTOS_THROUGHPUT;
+   setsockopt(sock, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int));
+#endif
+}
 void NetAccess::ReuseAddress(int sock)
 {
    setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,(char*)&one,sizeof(one));
@@ -143,7 +184,6 @@ void NetAccess::SetSocketMaxseg(int sock,int socket_maxseg)
       Log::global->Format(1,"setsockopt(TCP_MAXSEG,%d): %s\n",socket_maxseg,strerror(errno));
 #endif
 }
-
 void  NetAccess::SetSocketBuffer(int sock)
 {
    SetSocketBuffer(sock,socket_buffer);
@@ -192,6 +232,45 @@ socklen_t NetAccess::SocketAddrLen(const sockaddr_u *u)
    return sizeof(*u);
 }
 
+#if HAVE_INET_ATON && !HAVE_DECL_INET_ATON
+CDECL int inet_aton(const char *,struct in_addr *);
+#endif
+
+int NetAccess::SocketCreate(int af,int type,int proto)
+{
+   int s=socket(af,type,proto);
+   if(s<0)
+      return s;
+   const char *b=0;
+   sockaddr_u bind_addr;
+   memset(&bind_addr,0,sizeof(bind_addr));
+   bind_addr.in.sin_family=af;
+   if(af==AF_INET)
+   {
+      b=ResMgr::Query("net:socket-bind-ipv4",hostname);
+      if(!(b && b[0] && inet_aton(b,&bind_addr.in.sin_addr)))
+	 b=0;
+   }
+#if INET6
+   else if(af==AF_INET6)
+   {
+      b=ResMgr::Query("net:socket-bind-ipv6",hostname);
+      if(!(b && b[0] && inet_pton(af,b,&bind_addr.in6.sin6_addr)))
+	 b=0;
+   }
+#endif
+   if(b)
+   {
+      int res=bind(s,&bind_addr.sa,sizeof(bind_addr.sa));
+      if(res==-1)
+	 Log::global->Format(0,"**** bind(socket, %s): %s\n",b,strerror(errno));
+   }
+   return s;
+}
+int NetAccess::SocketCreateTCP(int af)
+{
+   return SocketCreate(af,SOCK_STREAM,IPPROTO_TCP);
+}
 int NetAccess::SocketConnect(int fd,const sockaddr_u *u)
 {
    // some systems have wrong connect() prototype, so we have to cast off const.
@@ -202,6 +281,63 @@ int NetAccess::SocketConnect(int fd,const sockaddr_u *u)
    return res;
 }
 
+#ifdef TIOCOUTQ
+static bool TIOCOUTQ_returns_free_space;
+static bool TIOCOUTQ_works;
+static bool TIOCOUTQ_tested;
+static void test_TIOCOUTQ()
+{
+   int sock=socket(PF_INET,SOCK_STREAM,IPPROTO_TCP);
+   if(sock==-1)
+      return;
+   TIOCOUTQ_tested=true;
+   int avail=-1;
+   socklen_t len=sizeof(avail);
+   if(getsockopt(sock,SOL_SOCKET,SO_SNDBUF,(char*)&avail,&len)==-1)
+      avail=-1;
+   int buf=-1;
+   if(ioctl(sock,TIOCOUTQ,&buf)==-1)
+      buf=-1;
+   if(buf>=0 && avail>0 && (buf==0 || buf==avail))
+   {
+      TIOCOUTQ_works=true;
+      TIOCOUTQ_returns_free_space=(buf==avail);
+   }
+   close(sock);
+}
+#endif
+int NetAccess::SocketBuffered(int sock)
+{
+#ifdef TIOCOUTQ
+   if(!TIOCOUTQ_tested)
+      test_TIOCOUTQ();
+   if(!TIOCOUTQ_works)
+      return 0;
+   int buffer=0;
+   if(TIOCOUTQ_returns_free_space)
+   {
+      socklen_t len=sizeof(buffer);
+      if(getsockopt(sock,SOL_SOCKET,SO_SNDBUF,(char*)&buffer,&len)==-1)
+	 return 0;
+      int avail=buffer;
+      if(ioctl(sock,TIOCOUTQ,&avail)==-1)
+	 return 0;
+      if(avail>buffer)
+	 return 0; // something wrong
+      buffer-=avail;
+      buffer=buffer*3/4; // approx...
+   }
+   else
+   {
+      if(ioctl(sock,TIOCOUTQ,&buffer)==-1)
+	 return 0;
+   }
+   return buffer;
+#else
+   return 0;
+#endif
+}
+
 void NetAccess::SayConnectingTo()
 {
    assert(peer_curr<peer_num);
@@ -236,6 +372,7 @@ void NetAccess::SetProxy(const char *px)
    proxy_port=xstrdup(url.port);
    proxy_user=xstrdup(url.user);
    proxy_pass=xstrdup(url.pass);
+   proxy_proto=xstrdup(url.proto);
    ClearPeer();
 }
 
@@ -451,25 +588,6 @@ void RateLimit::ReconfigTotal()
    total_reconfig_needed = false;
 }
 
-long NetAccess::ReconnectInterval()
-{
-   // cyclic exponential growth.
-   float interval = reconnect_interval;
-   if(reconnect_interval_multiplier>1
-   && reconnect_interval_max>=reconnect_interval*reconnect_interval_multiplier
-   && retries>0)
-   {
-      int modval = (int)(log((float)reconnect_interval_max/reconnect_interval)
-                              / log(reconnect_interval_multiplier) + 1.999);
-
-      interval *= pow(reconnect_interval_multiplier, (retries-1)%modval);
-
-      if( interval > reconnect_interval_max )
-         interval = reconnect_interval_max;
-   }
-   return long(interval+.5);
-}
-
 bool NetAccess::ReconnectAllowed()
 {
    if(max_retries>0 && retries>=max_retries)
@@ -478,10 +596,9 @@ bool NetAccess::ReconnectAllowed()
       return false;
    if(try_time==0)
       return true;
-   long interval = ReconnectInterval();
-   if(time_t(now) >= try_time+interval)
+   if(time_t(now) >= try_time+long(reconnect_interval_current))
       return true;
-   TimeoutS(interval-(time_t(now)-try_time));
+   TimeoutS(long(reconnect_interval_current)-(time_t(now)-try_time));
    return false;
 }
 
@@ -490,7 +607,7 @@ const char *NetAccess::DelayingMessage()
    static char buf[80];
    if(connection_limit>0 && connection_limit<=CountConnections())
       return _("Connection limit reached");
-   long remains=ReconnectInterval()-(time_t(now)-try_time);
+   long remains=long(reconnect_interval_current)-(time_t(now)-try_time);
    if(remains<=0)
       return "";
    sprintf(buf,"%s: %ld",_("Delaying before reconnect"),remains);
@@ -500,17 +617,29 @@ const char *NetAccess::DelayingMessage()
 
 bool NetAccess::NextTry()
 {
-   try_time=now;
-
    if(max_retries>0 && retries>=max_retries)
    {
       Fatal(_("max-retries exceeded"));
       return false;
    }
+   if(retries==0)
+      reconnect_interval_current=reconnect_interval;
+   else if(reconnect_interval_multiplier>1)
+   {
+      reconnect_interval_current*=reconnect_interval_multiplier;
+      if(reconnect_interval_current>reconnect_interval_max)
+	 reconnect_interval_current=reconnect_interval_max;
+   }
+   try_time=now;
    retries++;
-
    return true;
 }
+void NetAccess::TrySuccess()
+{
+   retries=0;
+   persist_retries=0;
+   reconnect_interval_current=reconnect_interval;
+}
 
 void NetAccess::Close()
 {
@@ -520,8 +649,7 @@ void NetAccess::Close()
       TimeoutS(idle);
    }
 
-   retries=0;
-   persist_retries=0;
+   TrySuccess();
 
    Delete(resolver);
    resolver=0;
@@ -540,6 +668,32 @@ int NetAccess::CountConnections()
    return count;
 }
 
+void NetAccess::PropagateHomeAuto()
+{
+   if(!home_auto)
+      return;
+   for(FA *fo=FirstSameSite(); fo!=0; fo=NextSameSite(fo))
+   {
+      NetAccess *o=(NetAccess*)fo; // we are sure it is NetAccess.
+      if(!o->home_auto)
+      {
+	 o->home_auto=xstrdup(home_auto);
+	 if(!o->home)
+	    o->set_home(home_auto);
+      }
+   }
+}
+const char *NetAccess::FindHomeAuto()
+{
+   for(FA *fo=FirstSameSite(); fo!=0; fo=NextSameSite(fo))
+   {
+      NetAccess *o=(NetAccess*)fo; // we are sure it is NetAccess.
+      if(o->home_auto)
+	 return o->home_auto;
+   }
+   return 0;
+}
+
 
 // GenericParseListInfo implementation
 int GenericParseListInfo::Do()
@@ -552,6 +706,7 @@ int GenericParseListInfo::Do()
    int res;
    int m=STALL;
 
+do_again:
    if(done)
       return m;
 
@@ -559,10 +714,21 @@ int GenericParseListInfo::Do()
    {
       const char *cache_buffer=0;
       int cache_buffer_size=0;
-      if(use_cache && LsCache::Find(session,"",mode,
+      int err;
+      if(use_cache && LsCache::Find(session,"",mode,&err,
 				    &cache_buffer,&cache_buffer_size))
       {
-	 ubuf=new Buffer();
+	 if(err)
+	 {
+	    if(mode==FA::MP_LIST)
+	    {
+	       mode=FA::LONG_LIST;
+	       goto do_again;
+	    }
+	    SetErrorCached(cache_buffer);
+	    return MOVED;
+	 }
+	 ubuf=new IOBuffer(IOBuffer::GET);
 	 ubuf->Put(cache_buffer,cache_buffer_size);
 	 ubuf->PutEOF();
       }
@@ -574,6 +740,8 @@ int GenericParseListInfo::Do()
 	 ubuf->SetSpeedometer(new Speedometer());
 	 if(LsCache::IsEnabled())
 	    ubuf->Save(LsCache::SizeLimit());
+	 Roll(session);
+	 Roll(ubuf);
       }
       m=MOVED;
    }
@@ -581,6 +749,15 @@ int GenericParseListInfo::Do()
    {
       if(ubuf->Error())
       {
+	 LsCache::Add(session,"",mode,session->GetErrorCode(),ubuf);
+	 if(mode==FA::MP_LIST)
+	 {
+	    mode=FA::LONG_LIST;
+	    delete ubuf;
+	    ubuf=0;
+	    m=MOVED;
+	    goto do_again;
+	 }
 	 SetError(ubuf->ErrorText());
 	 Delete(ubuf);
 	 ubuf=0;
@@ -590,7 +767,7 @@ int GenericParseListInfo::Do()
       if(!ubuf->Eof())
 	 return m;
 
-      LsCache::Add(session,"",mode,ubuf);
+      LsCache::Add(session,"",mode,FA::OK,ubuf);
 
       // now we have all the index in ubuf; parse it.
       const char *b;
@@ -600,13 +777,28 @@ int GenericParseListInfo::Do()
       int old_mode=mode;
 
       FileSet *set=Parse(b,len);
-      if(result)
+      if(set)
       {
-	 result->Merge(set);
-	 delete set;
+	 set->rewind();
+	 for(file=set->curr(); file!=0; file=set->next())
+	 {
+	    // tilde is special.
+	    if(file->name[0]=='~')
+	    {
+	       int len=strlen(file->name);
+	       file->name=(char*)xrealloc(file->name,len+3);
+	       memmove(file->name+2,file->name,len+1);
+	       memcpy(file->name,"./",2);
+	    }
+	 }
+	 if(result)
+	 {
+	    result->Merge(set);
+	    delete set;
+	 }
+	 else
+	    result=set;
       }
-      else
-	 result=set;
 
       Delete(ubuf);
       ubuf=0;
@@ -636,6 +828,8 @@ int GenericParseListInfo::Do()
       result->rewind();
       for(file=result->curr(); file!=0; file=result->next())
       {
+	 cur->size=NO_SIZE;
+	 cur->time=NO_DATE;
 	 cur->get_size = need_size && !(file->defined & file->SIZE);
 	 cur->get_time = need_time && (!(file->defined & file->DATE)
 				 || (file->date_prec>0 && can_get_prec_time));
@@ -645,8 +839,8 @@ int GenericParseListInfo::Do()
 	 {
 	    if(file->filetype==file->SYMLINK && follow_symlinks)
 	    {
-	       file->filetype=file->NORMAL;
-	       file->defined &= ~(file->SIZE|file->SYMLINK_DEF|file->MODE|file->DATE);
+	       //file->filetype=file->NORMAL;
+	       file->defined &= ~(file->SIZE|file->SYMLINK_DEF|file->MODE|file->DATE|file->TYPE);
 	       cur->get_size=true;
 	       cur->get_time=true;
 	    }
@@ -693,15 +887,12 @@ int GenericParseListInfo::Do()
 	 return m;
       if(res==FA::IN_PROGRESS)
 	 return m;
-      assert(res==FA::OK);
       session->Close();
-
-      cur=get_info;
       for(cur=get_info; get_info_cnt-->0; cur++)
       {
-	 if(cur->time!=(time_t)-1)
+	 if(cur->time!=NO_DATE)
 	    result->SetDate(cur->file,cur->time,0);
-	 if(cur->size!=-1)
+	 if(cur->size!=NO_SIZE)
 	    result->SetSize(cur->file,cur->size);
       }
       xfree(get_info);
@@ -723,7 +914,7 @@ GenericParseListInfo::GenericParseListInfo(FileAccess *s,const char *p)
    get_time_for_dirs=true;
    can_get_prec_time=true;
 
-   mode=FA::LONG_LIST;
+   mode=FA::MP_LIST;
 }
 
 GenericParseListInfo::~GenericParseListInfo()
