@@ -268,7 +268,7 @@ OutputJob::~OutputJob()
    delete output_fd;
    SessionPool::Reuse(fa);
    xfree(fa_path);
-   Delete(tmp_buf);
+   delete tmp_buf;
 
    xfree(a0);
    xfree(filter);
@@ -311,6 +311,10 @@ bool OutputJob::ShowStatusLine(StatusLine *s)
       return true;
    }
 
+   /* Don't disable write if there are data to be written in buffer */
+   if(output->GetCopy()->WriteAllowed() && output->GetCopy()->WritePending())
+      return false;
+
    /* There hasn't been output in a while.  Stop the output again,
     * so the FileCopy will clear the StatusLine when there's more data. */
    output->GetCopy()->AllowWrite(false);
@@ -337,6 +341,9 @@ const char *OutputJob::Status(const StatusLine *s)
 
 void OutputJob::PutEOF()
 {
+   if(Error())
+      return;
+
    /* Make sure we've sent at least one (empty) block.  This ensures
     * that we always start the input->output code path. */
    Put("", 0);
@@ -457,22 +464,20 @@ void OutputJob::Bg()
    super::Bg();
 }
 
-void OutputJob::Suspend()
+void OutputJob::SuspendInternal()
 {
    if(input)
-      input->Suspend();
+      input->SuspendSlave();
    if(output && input != output)
-      output->Suspend();
-   super::Suspend();
+      output->SuspendSlave();
 }
 
-void OutputJob::Resume()
+void OutputJob::ResumeInternal()
 {
    if(input)
-      input->Resume();
+      input->ResumeSlave();
    if(output && input != output)
-      output->Resume();
-   super::Resume();
+      output->ResumeSlave();
 }
 
 bool OutputJob::Full()
@@ -529,12 +534,12 @@ void OutputJob::Put(const char *buf,int size)
 	 Put(b,s);
       if(saved_buf->Eof())
 	 PutEOF();
-      Delete(saved_buf);
+      delete saved_buf;
    }
 
    update_timer.SetResource("cmd:status-interval",0);
 
-   int oldpos = InputPeer()->GetPos();
+   off_t oldpos = InputPeer()->GetPos();
    InputPeer()->Put(buf, size);
    InputPeer()->SetPos(oldpos);
 }
