@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ProcWait.cc,v 1.13 2002/01/21 16:25:15 lav Exp $ */
+/* $Id: ProcWait.cc,v 1.17 2005/10/17 11:39:32 lav Exp $ */
 
 #include <config.h>
 #include <sys/wait.h>
@@ -49,18 +49,15 @@ int ProcWait::Do()
    {
       if(status!=RUNNING)
 	 return MOVED;
-      if(NonFatalError(errno))
-	 goto leave;
       // waitpid failed, check the process existence
       if(kill(pid,0)==-1)
       {
 	 status=TERMINATED;
 	 term_info=255;
-	 return MOVED;
+	 m=MOVED;
+	 goto final;
       }
-      perror("waitpid");
-      TimeoutS(30);
-      return STALL;
+      goto leave;
    }
    if(res==pid)
    {
@@ -84,6 +81,8 @@ bool ProcWait::handle_info(int info)
    }
    else
    {
+      if(WIFSIGNALED(info) && WTERMSIG(info)==SIGINT)
+	 SignalHook::IncreaseCount(SIGINT);
       status=TERMINATED;
       term_info=info;
       return true;
@@ -160,6 +159,6 @@ void ProcWait::Signal(bool yes)
 void ProcWait::DeleteAll()
 {
    Signal(false);
-   while(chain)
-      Delete(chain);
+   for(ProcWait *scan=chain; scan; scan=scan->next)
+      scan->deleting=true;
 }
