@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.cc,v 1.53 2004/05/28 07:57:05 lav Exp $ */
+/* $Id: ResMgr.cc,v 1.58 2005/12/27 12:02:38 lav Exp $ */
 
 #include <config.h>
 
@@ -647,7 +647,7 @@ ResDecl::~ResDecl()
 #define HOUR   (60*MINUTE)
 #define DAY    (24*HOUR)
 
-TimeInterval::TimeInterval(const char *s)
+void TimeInterval::init(const char *s)
 {
    double interval=0;
    infty=false;
@@ -689,7 +689,7 @@ TimeInterval::TimeInterval(const char *s)
       error_text=_("Invalid time format. Format is <time><unit>, e.g. 2h30m.");
       return;
    }
-   Set(interval);
+   TimeDiff::Set(interval);
 }
 
 const char *ResMgr::TimeIntervalValidate(char **s)
@@ -702,31 +702,57 @@ const char *ResMgr::TimeIntervalValidate(char **s)
 
 Range::Range(const char *s)
 {
-   start=0;
-   end=0;
-   full=true;
+   start=end=0;
+   no_start=no_end=true;
    error_text=0;
 
    if(!strcasecmp(s,"full") || !strcasecmp(s,"any"))
       return;
 
-   if(sscanf(s,"%d-%d",&start,&end)!=2)
+   int n;
+   int len=strlen(s);
+   char start_k=0,end_k=0;
+   if(sscanf(s,"%lld-%n",&start,&n)==1 && n==len)
+      no_start=false;
+   else if(sscanf(s,"-%lld%n",&end,&n)==1 && n==len)
+      no_end=false;
+   else if(sscanf(s,"%lld-%lld%n",&start,&end,&n)==2 && n==len)
+      no_start=no_end=false;
+   else if(sscanf(s,"%lld%c-%n",&start,&start_k,&n)==2 && n==len)
+      no_start=false;
+   else if(sscanf(s,"-%lld%c%n",&end,&end_k,&n)==2 && n==len)
+      no_end=false;
+   else if(sscanf(s,"%lld%c-%lld%n",&start,&start_k,&end,&n)==3 && n==len)
+      no_start=no_end=false;
+   else if(sscanf(s,"%lld-%lld%c%n",&start,&end,&end_k,&n)==3 && n==len)
+      no_start=no_end=false;
+   else if(sscanf(s,"%lld%c-%lld%c%n",&start,&start_k,&end,&end_k,&n)==4 && n==len)
+      no_start=no_end=false;
+   else
    {
       error_text=_("Invalid range format. Format is min-max, e.g. 10-20.");
       return;
    }
-   if(start<end)
-   {
-      int tmp=start;
-      start=end;
-      end=tmp;
-   }
-   full=false;
+   if(start_k)
+      error_text=scale(&start,start_k);
+   if(!error_text && end_k)
+      error_text=scale(&end,end_k);
+   if(!no_start && !no_end && start>end)
+      start=replace_value(end,start);
 }
 
-bool Range::Match(int n)
+const char *Range::scale(long long *value,char suf)
 {
-   return full || (n>=start && n<=end);
+   // kilo, Mega, Giga, Tera, Peta, Exa, Zetta, Yotta
+   static const char s[]="kMGTPEZY";
+   const char *match=strchr(s,suf);
+   if(!match)
+      return _("Invalid suffix. Valid suffixes are: k, M, G, T, P, E, Z, Y");
+   unsigned shift=10*(match-s+1);
+   if(((*value<<shift)>>shift)!=*value)
+      return _("Integer overflow");
+   *value<<=shift;
+   return 0;
 }
 
 #if !HAVE_DECL_RANDOM
@@ -734,7 +760,7 @@ CDECL long random();
 CDECL void srandom(unsigned seed);
 #endif
 
-int Range::Random()
+long long Range::Random()
 {
    static bool init=false;
    if(!init)
@@ -743,12 +769,14 @@ int Range::Random()
       init=true;
    }
 
-   if(full)
+   if(no_start && no_end)
       return random();
+   if(no_end)
+      return start+random();
 
    // interval [0;1)
    double mult=random()/2147483648.0;
-   return start + (int)((end-start+1)*mult);
+   return start + (long long)((end-start+1)*mult);
 }
 
 const char *ResMgr::RangeValidate(char **s)
@@ -801,34 +829,7 @@ const char *ResMgr::IPv6AddrValidate(char **value)
 }
 #endif
 
-const char *ResMgr::FileReadable(char **value)
-{
-   if(!**value)
-      return 0;
-   const char *f=expand_home_relative(*value);
-   char *cwd=0;
-   const char *error=0;
-   if(f[0]!='/')
-   {
-      cwd=xgetcwd();
-      if(cwd)
-	 f=dir_file(cwd,f);
-   }
-   if(access(f,R_OK)<0)
-      error=strerror(errno);
-   else
-   {
-      if(f!=*value)
-      {
-	 xfree(*value);
-	 *value=xstrdup(f);
-      }
-   }
-   xfree(cwd);
-   return error;
-}
-
-const char *ResMgr::DirReadable(char **value)
+const char *ResMgr::FileAccessible(char **value,int mode,int want_dir)
 {
    if(!**value)
       return 0;
@@ -844,9 +845,9 @@ const char *ResMgr::DirReadable(char **value)
    struct stat st;
    if(stat(f,&st)<0)
       error=strerror(errno);
-   else if(!S_ISDIR(st.st_mode))
-      error=strerror(ENOTDIR);
-   else if(access(f,R_OK|X_OK)<0)
+   else if(want_dir ^ S_ISDIR(st.st_mode))
+      error=strerror(want_dir?ENOTDIR:EISDIR);
+   else if(access(f,mode)<0)
       error=strerror(errno);
    else
    {
@@ -857,7 +858,19 @@ const char *ResMgr::DirReadable(char **value)
       }
    }
    xfree(cwd);
-   return 0;
+   return error;
+}
+const char *ResMgr::FileReadable(char **value)
+{
+   return FileAccessible(value,R_OK);
+}
+const char *ResMgr::FileExecutable(char **value)
+{
+   return FileAccessible(value,X_OK);
+}
+const char *ResMgr::DirReadable(char **value)
+{
+   return FileAccessible(value,R_OK|X_OK,1);
 }
 
 CDECL_BEGIN
