@@ -36,9 +36,11 @@ class SMTask
    static int stack_ptr;
    static int stack_size;
 
+   bool	 suspended;
+   bool	 suspended_slave;
+
 protected:
    int	 running;
-   bool	 suspended;
    bool	 deleting;
 
    enum
@@ -50,6 +52,10 @@ protected:
 
    virtual ~SMTask();
 
+   // SuspendInternal and ResumeInternal usually suspend and resume slave tasks
+   virtual void SuspendInternal() {}
+   virtual void ResumeInternal() {}
+
 public:
    PollVec  block;
 
@@ -61,11 +67,18 @@ public:
    static void UpdateNow() { now.SetToCurrentTime(); }
 
    static void Schedule();
+   static int CollectGarbage();
    static void Block() { sched_total.Block(); }
 
-   virtual void Suspend();
-   virtual void Resume();
-   bool IsSuspended() { return suspended; }
+   void Suspend();
+   void Resume();
+
+   // SuspendSlave and ResumeSlave are used in SuspendInternal/ResumeInternal
+   // to suspend/resume slave tasks
+   void SuspendSlave();
+   void ResumeSlave();
+
+   bool IsSuspended() { return suspended|suspended_slave; }
 
    virtual void Reconfig(const char *name=0) {};
    static void ReconfigAll(const char *name);
@@ -74,9 +87,9 @@ public:
 
    SMTask();
 
+   void DeleteLater() { deleting=true; }
    static void Delete(SMTask *);
    static int Roll(SMTask *);
-   static void DeleteAll();
    static void RollAll(int max_time);
 
    static SMTask *current;
