@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: bookmark.cc,v 1.8.2.1 2002/01/15 16:20:23 lav Exp $ */
+/* $Id: bookmark.cc,v 1.16 2005/12/27 06:25:50 lav Exp $ */
 
 #include <config.h>
 #include "bookmark.h"
@@ -26,18 +26,26 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include "trio.h"
+#include "ResMgr.h"
+#include "misc.h"
+#include "url.h"
 
 #define super KeyValueDB
 
+Bookmark lftp_bookmarks;
+
+ResDecl res_auto_sync("bmk:auto-sync","yes",ResMgr::BoolValidate,ResMgr::NoClosure);
+
 Bookmark::Bookmark()
 {
-   const char *home=getenv("HOME");
-   if(home==0)
-      home="";
-   const char *add="/.lftp/bookmarks";
-   bm_file=xstrdup(home,+strlen(add));
-   strcat(bm_file,add);
-
+   bm_file=0;
+   const char *home=get_lftp_home();
+   if(home)
+   {
+      const char *add="/bookmarks";
+      bm_file=xstrdup(home,+strlen(add));
+      strcat(bm_file,add);
+   }
    bm_fd=-1;
    stamp=(time_t)-1;
 }
@@ -50,6 +58,8 @@ Bookmark::~Bookmark()
 
 void Bookmark::Refresh()
 {
+   if(!bm_file)
+      return;
    struct stat st;
    if((bm_fd==-1 ? stat(bm_file,&st) : fstat(bm_fd,&st)) == -1)
       return;
@@ -61,6 +71,8 @@ void Bookmark::Refresh()
 void Bookmark::Load()
 {
    super::Empty();
+   if(!bm_file)
+      return;
    if(bm_fd==-1)
    {
       bm_fd=open(bm_file,O_RDONLY);
@@ -77,8 +89,17 @@ void Bookmark::Load()
    super::Read(dup(bm_fd));   // Read closes file
 }
 
+
+static bool auto_sync;
 void Bookmark::PreModify()
 {
+   if(!bm_file)
+      return;
+
+   auto_sync=ResMgr::QueryBool("bmk:auto-sync",0);
+   if(!auto_sync)
+      return;
+
    Close();
    bm_fd=open(bm_file,O_RDWR|O_CREAT,0600);
    if(bm_fd==-1)
@@ -91,22 +112,44 @@ void Bookmark::PreModify()
    }
    Refresh();
 }
-
 void Bookmark::PostModify()
 {
+   if(!bm_file)
+      return;
+
+   if(!auto_sync)
+      return;
+
    // the file is already locked in PreModify.
    lseek(bm_fd,0,SEEK_SET);
 
 #ifdef HAVE_FTRUNCATE
-   ftruncate(bm_fd,0);
-#else
-   close(open(bm_file,O_WRONLY|O_TRUNC));
+   if(ftruncate(bm_fd,0)==-1) // note the following statement
 #endif
+   close(open(bm_file,O_WRONLY|O_TRUNC));
 
    super::Write(bm_fd);
    bm_fd=-1;   // Write closes file
 }
 
+void Bookmark::UserSave()
+{
+   if(!bm_file)
+      return;
+   Close();
+   bm_fd=open(bm_file,O_RDWR|O_CREAT|O_TRUNC,0600);
+   if(bm_fd==-1)
+      return;
+   if(Lock(F_WRLCK)==-1)
+   {
+      fprintf(stderr,"%s: lock for writing failed\n",bm_file);
+      Close();
+      return;
+   }
+   super::Write(bm_fd);
+   bm_fd=-1;   // Write closes file
+}
+
 void Bookmark::Add(const char *key,const char *value)
 {
    PreModify();
@@ -130,10 +173,18 @@ void Bookmark::Close()
    }
 }
 
+void Bookmark::AutoSync()
+{
+   if(ResMgr::QueryBool("bmk:auto-sync",0))
+   {
+      Refresh();
+      Close();
+   }
+}
+
 const char *Bookmark::Lookup(const char *key)
 {
-   Refresh();
-   Close();
+   AutoSync();
    return super::Lookup(key);
 }
 
@@ -148,7 +199,12 @@ void Bookmark::List()
 
 char *Bookmark::Format()
 {
-   Refresh();
-   Close();
+   AutoSync();
    return super::Format();
 }
+
+char *Bookmark::FormatHidePasswords()
+{
+   AutoSync();
+   return super::Format(url::hide_password);
+}
