@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,27 +18,33 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: buffer.h,v 1.21 2001/12/18 10:50:39 lav Exp $ */
+/* $Id: buffer.h,v 1.38 2005/11/17 13:03:39 lav Exp $ */
 
 #ifndef BUFFER_H
 #define BUFFER_H
 
 #include "SMTask.h"
 #include "Filter.h"
+#include "Timer.h"
 #include "fg.h"
 #include "xstring.h"
 
 #include <stdarg.h>
+CDECL_BEGIN
+#include <iconv.h>
+CDECL_END
 
-#define GET_BUFSIZE 0x4000
+#define GET_BUFSIZE 0x10000
+#define PUT_LL_MIN  0x2000
 
 class Speedometer;
 
-class Buffer : public SMTask
+class Buffer
 {
 protected:
    char *error_text;
    int  saved_errno;
+   bool error_fatal;
 
    char *buffer;
    int buffer_allocated;
@@ -63,19 +69,16 @@ protected:
 
    void SaveMaxCheck(int addsize);
 
-   virtual ~Buffer();
-
 public:
-   virtual int Do();
-   virtual bool Done() { return in_buffer==0; }
-   bool Error() { return error_text!=0; }
-   void SetError(const char *e);
-   void SetError2(const char *e1,const char *e2);
-   int  Errno() { return saved_errno; }
-   const char *ErrorText() { return error_text; }
-   int Size() { return in_buffer; }
-   bool Eof() { return eof; }
-   bool Broken() { return broken; }
+   bool Error() const { return error_text!=0; }
+   bool ErrorFatal() const { return error_fatal; }
+   void SetError(const char *e,bool fatal=false);
+   void SetError2(const char *e1,const char *e2,bool fatal=false);
+   void SetErrorCached(const char *e);
+   const char *ErrorText() const { return error_text; }
+   int Size() const { return in_buffer; }
+   bool Eof() const { return eof; }
+   bool Broken() const { return broken; }
 
    void Get(const char **buf,int *size);
    void Skip(int len); // Get(); consume; Skip()
@@ -86,8 +89,27 @@ public:
    void vFormat(const char *f, va_list v);
    void PutEOF() { eof=true; PutEOF_LL(); }
 
+   unsigned long long UnpackUINT64BE(int offset=0);
+   unsigned UnpackUINT32BE(int offset=0);
+   unsigned UnpackUINT16BE(int offset=0);
+   unsigned UnpackUINT8(int offset=0);
+   void PackUINT64BE(unsigned long long data);
+   void PackUINT32BE(unsigned data);
+   void PackUINT16BE(unsigned data);
+   void PackUINT8(unsigned data);
+
+   long long UnpackINT64BE(int offset=0);
+   int UnpackINT32BE(int offset=0);
+   int UnpackINT16BE(int offset=0);
+   int UnpackINT8(int offset=0);
+   void PackINT64BE(long long data);
+   void PackINT32BE(int data);
+   void PackINT16BE(int data);
+   void PackINT8(int data);
+
    // useful for cache.
    void Save(int m) { save=true; save_max=m; }
+   bool IsSaving() const { return save; }
    void GetSaved(const char **buf,int *size) const;
    void SaveRollback(int p);
 
@@ -99,55 +121,99 @@ public:
 
    void Empty();
 
-   virtual FgData *GetFgData(bool) { return 0; }
-
-   virtual time_t EventTime() { return now; }
-
    Buffer();
+   virtual ~Buffer();
 };
 
-class IOBuffer : public Buffer
+class DirectedBuffer : public Buffer
 {
 public:
    enum dir_t { GET, PUT };
 
 protected:
-   time_t event_time; // used to detect timeouts
+   iconv_t backend_translate;
+   Buffer *untranslated;
    dir_t mode;
+   void EmbraceNewData(int len);
 
 public:
-   IOBuffer(dir_t m)
+   DirectedBuffer(dir_t m) { mode=m; backend_translate=0; untranslated=0; }
+   ~DirectedBuffer();
+   void SetTranslation(const char *be_encoding,bool translit=true);
+   virtual void PutTranslated(const char *buf,int size);
+   void PutTranslated(const char *buf) { PutTranslated(buf,strlen(buf)); }
+   void ResetTranslation();
+   void Put(const char *buf,int size)
+      {
+	 if(mode==PUT)
+	    PutTranslated(buf,size);
+	 else
+	    Buffer::Put(buf,size);
+      }
+   void Put(const char *buf) { Put(buf,strlen(buf)); }
+   dir_t GetDirection() { return mode; }
+};
+
+class IOBuffer : public DirectedBuffer, public SMTask
+{
+protected:
+   time_t event_time; // used to detect timeouts
+
+public:
+   IOBuffer(dir_t m) : DirectedBuffer(m)
       {
 	 event_time=now;
-	 mode=m;
       }
-   time_t EventTime()
+   virtual time_t EventTime()
       {
-	 if(suspended)
+	 if(IsSuspended())
 	    return now;
 	 return event_time;
       }
-   bool Done()
+   virtual bool Done()
       {
 	 return(broken || Error() || (eof && (mode==GET || in_buffer==0)));
       }
+   virtual int Get_LL(int size) { return 0; }
+   virtual int Do();
+
+   virtual FgData *GetFgData(bool) { return 0; }
+   virtual const char *Status() { return ""; }
+   virtual int Buffered() { return Size(); }
+};
+
+class IOBufferStacked : public IOBuffer
+{
+   IOBuffer *down;
+
+   int Get_LL(int size);
+   int Put_LL(const char *buf,int size);
+
+public:
+   IOBufferStacked(IOBuffer *b) : IOBuffer(b->GetDirection()) { down=b; }
+   ~IOBufferStacked() { Delete(down); }
+   time_t EventTime() { return down->EventTime(); }
+   int Do();
+   bool Done();
 };
 
 class IOBufferFDStream : public IOBuffer
 {
    FDStream *stream;
+   class Timer *put_ll_timer;
 
    int Get_LL(int size);
    int Put_LL(const char *buf,int size);
 
 protected:
-   ~IOBufferFDStream() { delete stream; }
+   ~IOBufferFDStream() { delete stream; delete put_ll_timer; }
 
 public:
-   IOBufferFDStream(FDStream *o,dir_t m) : IOBuffer(m) { stream=o; }
-   int Do();
+   IOBufferFDStream(FDStream *o,dir_t m,Timer *t=0)
+      : IOBuffer(m) { stream=o; put_ll_timer=t; }
    bool Done();
    FgData *GetFgData(bool fg);
+   const char *Status() { return stream->status; }
 };
 
 class FileAccess;
@@ -163,10 +229,10 @@ protected:
 
 public:
    IOBufferFileAccess(FileAccess *i) : IOBuffer(GET) { session=i; }
-   int Do();
 
-   void Suspend();
-   void Resume();
+   void SuspendInternal();
+   void ResumeInternal();
+   const char *Status();
 };
 
 #endif // BUFFER_H
