@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.210 2004/05/28 11:48:12 lav Exp $ */
+/* $Id: commands.cc,v 1.236 2005/12/27 06:25:22 lav Exp $ */
 
 #include <config.h>
 
@@ -33,6 +33,9 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <assert.h>
+#ifdef HAVE_DLFCN_H
+# include <dlfcn.h>
+#endif
 
 #include "CmdExec.h"
 #include "GetJob.h"
@@ -568,6 +571,8 @@ Job *CmdExec::builtin_cd()
    if(args->count()==1)
       args->Append("~");
 
+   bool is_file=false;
+
    if(args->count()!=2)
    {
       // xgettext:c-format
@@ -576,9 +581,7 @@ Job *CmdExec::builtin_cd()
    }
 
    const char *dir=args->getarg(1);
-
-   if(url::is_url(dir))
-      return builtin_open();
+   const char *url=0;
 
    if(!strcmp(dir,"-"))
    {
@@ -591,19 +594,41 @@ Job *CmdExec::builtin_cd()
       args->setarg(1,dir); // for status line
    }
 
-   xfree(old_cwd);
-   old_cwd=xstrdup(session->GetCwd());
+   if(url::is_url(dir))
+   {
+      ParsedURL u(dir,true);
+      FileAccess *new_session=FileAccess::New(&u);
+      bool same_site=session->SameSiteAs(new_session);
+      Delete(new_session);
+      if(!same_site)
+	 return builtin_open();
+      url=dir;
+      dir=alloca_strdup(u.path);
+      if(url::dir_needs_trailing_slash(u.proto))
+	 is_file=(last_char(dir)!='/');
+   }
+   else
+   {
+      if(url::dir_needs_trailing_slash(session->GetProto()))
+	 is_file=(last_char(dir)!='/');
+   }
 
+   old_cwd=session->GetCwd();
+   FileAccess::Path new_cwd=old_cwd;
+   new_cwd.Change(dir,is_file);
+   if(url)
+      new_cwd.SetURL(url);
    if(!verify_path || background
    || (!verify_path_cached && LsCache::IsDirectory(session,dir)==1))
    {
-      session->Chdir(dir,false);
+      cwd_history.Set(session,old_cwd);
+      session->SetCwd(new_cwd);
       if(slot)
-	 ConnectionSlot::SetCwd(slot,session->GetCwd());
+	 ConnectionSlot::SetCwd(slot,new_cwd);
       exit_code=0;
       return 0;
    }
-   session->Chdir(dir);
+   session->PathVerify(new_cwd);
    Roll(session);
    builtin=BUILTIN_CD;
    return this;
@@ -673,10 +698,16 @@ Job *CmdExec::builtin_lftp()
 	 cmd=acmd;
 	 break;
       case('c'):
-	 acmd=string_alloca(4+strlen(optarg));
-	 sprintf(acmd,"%s\n\n",optarg);
+      {
+	 int cmd_start=args->getindex()-1;
+	 char *c = (args->count()==cmd_start+1
+		      ? args->Combine(cmd_start) : args->CombineQuoted(cmd_start));
+	 acmd=string_alloca(4+strlen(c));
+	 sprintf(acmd,"%s\n\n",c);
 	 cmd=acmd;
+	 xfree(c);
 	 break;
+      }
       case('d'):
 	 debug=true;
 	 break;
@@ -733,10 +764,11 @@ Job *CmdExec::builtin_open()
       {"execute",required_argument,0,'e'},
       {"debug",optional_argument,0,'d'},
       {"no-bookmark",no_argument,0,'B'},
+      {"help",no_argument,0,'h'},
       {0,0,0,0}
    };
 
-   while((c=args->getopt_long("u:p:e:dB",open_options,0))!=EOF)
+   while((c=args->getopt_long("u:p:e:dBh",open_options,0))!=EOF)
    {
       switch(c)
       {
@@ -765,6 +797,12 @@ Job *CmdExec::builtin_open()
       case('B'):
 	 no_bm=true;
 	 break;
+      case('h'):
+	 if(!strcmp(op,"lftp"))
+	 {
+	    PrependCmd("help lftp");
+	    return 0;
+	 }
       case('?'):
 	 if(!strcmp(op,"lftp"))
 	    eprintf(_("Try `%s --help' for more information\n"),op);
@@ -820,23 +858,37 @@ Job *CmdExec::builtin_open()
       if(host && host[0])
       {
 	 url=new ParsedURL(host);
+	 bool no_proto=(!url->proto);
+
+	 if(no_proto && url->host)
+	 {
+	    const char *p=ResMgr::Query("cmd:default-protocol",url->host);
+	    if(!p)
+	       p="ftp";
+	    char *u=string_alloca(strlen(p)+3+strlen(host)+1);
+	    sprintf(u,"%s://%s",p,host);
+	    delete url;
+	    url=new ParsedURL(u);
+	 }
+	 if(user)
+	 {
+	    url->user=user;
+	    url->pass=pass;
+	    char *host1=url->Combine();
+	    delete url;
+	    url=new ParsedURL(host1);
+	    xfree(host1);
+	 }
 
 	 const ParsedURL &uc=*url;
-	 if(uc.host && uc.host[0])
+	 if(uc.host && uc.host[0] && uc.proto)
 	 {
 	    cwd_history.Set(session,session->GetCwd());
 
-	    FileAccess *new_session=0;
-
-	    const char *p=uc.proto;
-	    if(!p)
-	       p=ResMgr::Query("cmd:default-protocol",uc.host);
-	    if(!p)
-	       p="ftp";
-	    new_session=FileAccess::New(p,uc.host);
+	    FileAccess *new_session=FileAccess::New(uc.proto,uc.host);
 	    if(!new_session)
 	    {
-	       eprintf("%s: %s%s\n",args->a0(),p,
+	       eprintf("%s: %s%s\n",args->a0(),uc.proto,
 			_(" - not supported protocol"));
 	       return 0;
 	    }
@@ -860,7 +912,7 @@ Job *CmdExec::builtin_open()
 	 }
 
 	 // user gets substituted only if no proto is specified.
-	 if(!pass && (user || !uc.proto))
+	 if(!pass && (user || no_proto))
 	 {
 	    nrc=NetRC::LookupHost(host,user);
 	    if(nrc)
@@ -911,11 +963,29 @@ Job *CmdExec::builtin_open()
    {
       const char *old=cwd_history.Lookup(session);
       if(old)
-	 session->Chdir(old,false);
+      {
+	 bool is_file=false;
+	 const char *url=0;
+	 if(url::is_url(old))
+	 {
+	    ParsedURL u(old,true);
+	    url=old;
+	    old=alloca_strdup(u.path);
+	    if(url::dir_needs_trailing_slash(u.proto))
+	       is_file=(last_char(old)!='/');
+	 }
+	 else
+	 {
+	    if(url::dir_needs_trailing_slash(session->GetProto()))
+	       is_file=(last_char(old)!='/');
+	 }
+	 session->SetCwd(FileAccess::Path(old,is_file,url));
+      }
 
-      char *s=string_alloca(strlen(path)*2+40);
+      const char *cd_arg=(url && url->orig_url)?url->orig_url:path;
+      char *s=string_alloca(strlen(cd_arg)*2+40);
       strcpy(s,"&& cd \"");
-      unquote(s+strlen(s),path);
+      unquote(s+strlen(s),cd_arg);
       strcat(s,"\"");
       if(background)
 	 strcat(s,"&");
@@ -1030,7 +1100,7 @@ Job *CmdExec::builtin_queue()
 	  * it. */
 	 if(!isdigit((unsigned char)optarg[0]) || atoi(optarg) == 0)
 	 {
-	    eprintf(_("%s: -n: Number expected. "), args->a0());
+	    eprintf(_("%s: -n: positive number expected. "), args->a0());
 	    goto err;
 	 }
 	 /* make offsets match the jobs output (starting at 1) */
@@ -1087,7 +1157,7 @@ Job *CmdExec::builtin_queue()
 	       queue->Suspend();
 	    }
 	    else
-	       queue->PrintStatus(0,"");
+	       queue->PrintStatus(2,"");
 	    exit_code=0;
 	    break;
 	 }
@@ -1327,18 +1397,18 @@ const char *FileSetOutput::parse_argv(ArgV *a)
       {"dirsfirst", no_argument,0,'D'},
 
       {"sort", required_argument,0, 0},
+      {"reverse",no_argument,0,'r'},
       {"user", no_argument,0, 0},
       {"group", no_argument,0, 0},
       {"perms", no_argument,0, 0},
       {"date", no_argument,0, 0},
       {"linkcount", no_argument,0, 0},
       {"links", no_argument,0, 0},
-      {0, no_argument, 0, 'S'}, // sort by size
       {0,0,0,0}
    };
 
    int opt, longopt;
-   while((opt=a->getopt_long(":1BdFhiklqsDIS", cls_options, &longopt))!=EOF)
+   while((opt=a->getopt_long(":1BdFhiklqsDISrt", cls_options, &longopt))!=EOF)
    {
       switch(opt) {
       case 0:
@@ -1409,6 +1479,12 @@ const char *FileSetOutput::parse_argv(ArgV *a)
       case('S'):
 	 sort = FileSet::BYSIZE;
 	 break;
+      case('t'):
+	 sort = FileSet::BYDATE;
+	 break;
+      case('r'):
+	 sort_reverse = true;
+	 break;
 
       default:
 	 /* silly getopt won't give us its error instead of printing it, oh well.
@@ -1520,7 +1596,7 @@ CMD(get)
    bool glob=false;
    bool make_dirs=false;
    bool reverse=false;
-   char *output_dir=0;
+   const char *output_dir=0;
 
    if(!strncmp(op,"re",2))
    {
@@ -1529,7 +1605,7 @@ CMD(get)
    }
    if(!strcmp(op,"pget"))
    {
-      opts="+n:uO:";
+      opts="+n:euO:";
       n_conn=-1;
    }
    else if(!strcmp(op,"put") || !strcmp(op,"reput"))
@@ -1547,6 +1623,12 @@ CMD(get)
       opts="cEeadO:";
       reverse=true;
    }
+   if(!reverse)
+   {
+      output_dir=ResMgr::Query("xfer:destination-directory",session->GetHostName());
+      if(!*output_dir)
+	 output_dir=0;
+   }
    while((opt=args->getopt(opts))!=EOF)
    {
       switch(opt)
@@ -1594,7 +1676,8 @@ CMD(get)
 	 goto err;
       }
       delete get_args;
-      output_dir=xstrdup(output_dir);  // save it from deleting.
+      char *output_dir_dup=xstrdup(output_dir);  // save it from deleting.
+      output_dir=0;
       // remove options
       while(args->getindex()>1)
 	 args->delarg(1);
@@ -1605,8 +1688,8 @@ CMD(get)
 	 j->DeleteFiles();
       if(ascii)
 	 j->Ascii();
-      if(output_dir)
-	 j->OutputDir(output_dir);
+      if(output_dir_dup)
+	 j->OutputDir(output_dir_dup);
       args=0;
       return j;
    }
@@ -1649,6 +1732,8 @@ CMD(get)
       pgetJob *j=new pgetJob(session->Clone(),get_args);
       if(n_conn!=-1)
 	 j->SetMaxConn(n_conn);
+      if(del_target)
+	 j->RemoveTargetFirst();
       return j;
    }
 }
@@ -1707,12 +1792,12 @@ CMD(rm)
    if(args->getcurr()==0)
       goto print_usage;
 
-   rmJob *j=(rmdir?
-	     new rmdirJob(session->Clone(),args):
-	     new rmJob(session->Clone(),args));
+   rmJob *j=new rmJob(session->Clone(),args);
 
    if(recursive)
       j->Recurse();
+   if(rmdir)
+      j->Rmdir();
 
    args=0;
 
@@ -1761,7 +1846,17 @@ CMD(source)
    }
    else
       f=new FileStream(args->getarg(1),O_RDONLY|O_ASCII);
-
+   // try to open the file to return error code if failed, as FileFeeder
+   // cannot feed error codes.
+   if(f->getfd()==-1)
+   {
+      if(f->error())
+      {
+	 fprintf(stderr,"%s: %s\n",args->a0(),f->error_text);
+	 delete f;
+	 return 0;
+      }
+   }
    parent->SetCmdFeeder(new FileFeeder(f));
    exit_code=0;
    return 0;
@@ -2403,11 +2498,77 @@ CMD(help)
 CMD(ver)
 {
    printf(
-      _("Lftp | Version %s | Copyright (c) 1996-2004 Alexander V. Lukyanov\n"),VERSION);
+      _("LFTP | Version %s | Copyright (c) 1996-2005 Alexander V. Lukyanov\n"),VERSION);
+   printf("\n");
    printf(
-      _("This is free software with ABSOLUTELY NO WARRANTY. See COPYING for details.\n"));
+      _("LFTP is free software, covered by the GNU General Public License, and you are\n"
+      "welcome to change it and/or distribute copies of it under certain conditions.\n"
+      "There is absolutely no warranty for LFTP.  See COPYING for details.\n"));
+   printf("\n");
    printf(
       _("Send bug reports and questions to <%s>.\n"),"lftp@uniyar.ac.ru");
+
+#if defined(HAVE_DLOPEN) && defined(RTLD_DEFAULT)
+   printf("\n");
+   printf(_("Libraries used: "));
+
+   struct VersionInfo
+   {
+      const char *lib_name;
+      const char *symbol;
+      enum type_t { STRING_PTR, FUNC0, INT8_8 } type;
+      const char *skip_prefix;
+      typedef const char *(*func0)(int);
+      const char *query() const
+	 {
+	    static char ver[10];
+	    int v;
+	    void *sym_ptr=dlsym(RTLD_DEFAULT,symbol);
+	    if(!sym_ptr)
+	       return 0;
+	    const char *str=0;
+	    switch(type)
+	    {
+	    case STRING_PTR:
+	       str=*(const char**)sym_ptr;
+	       break;
+	    case FUNC0:
+	       str=((func0)sym_ptr)(0);
+	       break;
+	    case INT8_8:
+	       v=*(int*)sym_ptr;
+	       sprintf(ver,"%d.%d",(v>>8)&255,v&255);
+	       str=ver;
+	    }
+	    if(!str)
+	       return 0;
+	    if(skip_prefix && !strncmp(str,skip_prefix,strlen(skip_prefix)))
+	       str+=strlen(skip_prefix);
+	    return str;
+	 }
+   }
+   static const libs[]=
+   {
+      {"Readline",   "rl_library_version",   VersionInfo::STRING_PTR,0},
+      {"Expat",	     "XML_ExpatVersion",     VersionInfo::FUNC0,     "expat_"},
+      {"OpenSSL",    "SSL_version_str",	     VersionInfo::STRING_PTR,"OpenSSL "},
+      {"GnuTLS",     "gnutls_check_version", VersionInfo::FUNC0,     0},
+      {"libiconv",   "_libiconv_version",    VersionInfo::INT8_8,    0},
+      {0}
+   };
+
+   bool need_comma=false;
+   for(const VersionInfo *scan=libs; scan->lib_name; scan++)
+   {
+      const char *v=scan->query();
+      if(!v)
+	 continue;
+      printf("%s%s %s",need_comma?", ":"",scan->lib_name,v);
+      need_comma=true;
+   }
+   printf("\n");
+#endif // HAVE_DLOPEN
+
    exit_code=0;
    return 0;
 }
@@ -2438,7 +2599,7 @@ CMD(close)
 }
 
 const char * const bookmark_subcmd[]=
-   {"add","delete","list","edit","import",0};
+   {"add","delete","list","list-p","edit","import",0};
 static ResDecl res_save_passwords
    ("bmk:save-passwords","no",ResMgr::BoolValidate,0);
 
@@ -2463,9 +2624,9 @@ CMD(bookmark)
       return 0;
    }
 
-   if(!strcasecmp(op,"list"))
+   if(!strcasecmp(op,"list") || !strcasecmp(op,"list-p"))
    {
-      char *list=lftp_bookmarks.FormatHidePasswords();
+      char *list=op[4]?lftp_bookmarks.Format():lftp_bookmarks.FormatHidePasswords();
       OutputJob *out=new OutputJob(output, args->a0());
       Job *j=new echoJob(list,out);
       xfree(list);
@@ -2520,7 +2681,7 @@ CMD(bookmark)
    else if(!strcasecmp(op,"edit"))
    {
       lftp_bookmarks.Remove(""); // force bookmark file creation
-      parent->PrependCmd("shell \"/bin/sh -c 'exec ${EDITOR:-vi} $HOME/.lftp/bookmarks'\"\n");
+      parent->PrependCmd("shell \"/bin/sh -c 'exec ${EDITOR:-vi} ${LFTP_HOME:-$HOME/.lftp}/bookmarks'\"\n");
    }
    else if(!strcasecmp(op,"import"))
    {
@@ -2536,6 +2697,16 @@ CMD(bookmark)
 	 exit_code=0;
       }
    }
+   else if(!strcasecmp(op,"load"))
+   {
+      lftp_bookmarks.UserLoad();
+      exit_code=0;
+   }
+   else if(!strcasecmp(op,"save"))
+   {
+      lftp_bookmarks.UserSave();
+      exit_code=0;
+   }
    return 0;
 }
 
@@ -2603,8 +2774,7 @@ CMD(find)
 
    if(!args->getcurr())
       args->Append(".");
-   FinderJob_List *j=new class FinderJob_List(session->Clone(),args,
-      output?output:new FDStream(1,"<stdout>"));
+   FinderJob_List *j=new class FinderJob_List(session->Clone(),args,output);
    j->set_maxdepth(maxdepth);
    args=0;
    output=0;
@@ -2740,8 +2910,7 @@ CMD(du)
 
    if(!args->getcurr())
       args->Append(".");
-   FinderJob_Du *j=new class FinderJob_Du(session->Clone(),args,
-      output?output:new FDStream(1,"<stdout>"));
+   FinderJob_Du *j=new class FinderJob_Du(session->Clone(),args,output);
    args=0;
    j->PrintDepth(maxdepth);
    j->SetBlockSize(blocksize);
@@ -3116,6 +3285,7 @@ CMD(slot)
    if(n)
    {
       parent->ChangeSlot(n);
+      exit_code=0;
       return 0;
    }
    else
