@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ftpclass.h,v 1.124 2005/03/09 07:54:10 lav Exp $ */
+/* $Id: ftpclass.h,v 1.129 2005/12/27 12:22:43 lav Exp $ */
 
 #ifndef FTPCLASS_H
 #define FTPCLASS_H
@@ -28,8 +28,8 @@
 
 #include "NetAccess.h"
 
-#ifdef USE_SSL
-# include <openssl/ssl.h>
+#if USE_SSL
+# include "lftp_ssl.h"
 #endif
 
 class IOBufferTelnet : public IOBufferStacked
@@ -104,9 +104,8 @@ class Ftp : public NetAccess
       off_t last_rest;	// last successful REST position.
       off_t rest_pos;	// the number sent with REST command.
 
-#ifdef USE_SSL
-      SSL *control_ssl;
-      SSL *data_ssl;
+#if USE_SSL
+      lftp_ssl *control_ssl;
       char prot;  // current data protection scheme 'C'lear or 'P'rivate
       bool auth_sent;
       bool auth_supported;
@@ -132,6 +131,7 @@ class Ftp : public NetAccess
       void InitTelnetLayer();
       void SetControlConnectionTranslation(const char *cs);
 
+      void CloseDataSocket(); // only closes socket, does not delete iobuf.
       void CloseDataConnection();
       void AbortDataConnection();
       void CloseAbortedDataConnection();
@@ -181,8 +181,8 @@ class Ftp : public NetAccess
 	 LANG,
 	 SITE_UTIME,
 	 QUOTED		// check response for any command submitted by QUOTE_CMD
-#ifdef USE_SSL
-	 ,AUTH_TLS,PROT,SSCN
+#if USE_SSL
+	 ,AUTH_TLS,PROT,SSCN,CCC
 #endif
       };
 
@@ -269,8 +269,7 @@ class Ftp : public NetAccess
 
    void	 HandleTimeout();
 
-#ifdef USE_SSL
-   void	 BlockOnSSL(SSL*);
+#if USE_SSL
 protected:
    bool	 ftps;	  // ssl and prot='P' by default (port 990)
 private:
@@ -306,6 +305,7 @@ private:
    void SendAuth(const char *auth);
    void TuneConnectionAfterFEAT();
    void SendOPTS_MLST();
+   void SendPROT(char want_prot);
 
    const char *QueryStringWithUserAtHost(const char *);
 
