@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: history.cc,v 1.11 2003/12/17 13:00:03 lav Exp $ */
+/* $Id: history.cc,v 1.16 2005/12/27 06:25:50 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -28,6 +28,7 @@
 #include "trio.h"
 #include "history.h"
 #include "url.h"
+#include "misc.h"
 
 #define super KeyValueDB
 
@@ -37,13 +38,14 @@ History::History()
    stamp=0;
    fd=-1;
    modified=false;
-
-   const char *home=getenv("HOME");
-   if(home==0)
-      home="";
-   const char *add="/.lftp/cwd_history";
-   file=xstrdup(home,+strlen(add));
-   strcat(file,add);
+   file=0;
+   const char *home=get_lftp_home();
+   if(home)
+   {
+      const char *add="/cwd_history";
+      file=xstrdup(home,+strlen(add));
+      strcat(file,add);
+   }
 }
 
 History::~History()
@@ -70,7 +72,8 @@ const char *History::extract_url(const char *res)
       buf=(char*)xrealloc(buf,buf_len=len+64);
 
    strcpy(buf,url);
-   url::decode_string(buf);
+   if(!url::is_url(buf))
+      url::decode_string(buf);
 
    return buf;
 }
@@ -100,6 +103,8 @@ const char *History::Lookup(FileAccess *s)
 
 void History::Refresh()
 {
+   if(!file)
+      return;
    struct stat st;
    if((fd==-1 ? stat(file,&st) : fstat(fd,&st)) == -1)
       return;
@@ -112,6 +117,8 @@ void History::Load()
 {
    if(full)
       full->Empty();
+   if(!file)
+      return;
    if(fd==-1)
    {
       fd=open(file,O_RDONLY);
@@ -140,13 +147,20 @@ void History::Close()
    }
 }
 
-void History::Set(FileAccess *s,const char *cwd)
+void History::Set(FileAccess *s,const FileAccess::Path &cwd)
 {
-   if(cwd==0 || !strcmp(cwd,"~") || s->GetHostName()==0)
+   if(cwd.path==0 || !strcmp(cwd.path,"~") || s->GetHostName()==0)
       return;
-   char *res=(char*)alloca(32+strlen(cwd)*3+1);
+   char *res=(char*)alloca(32+strlen(cwd.path)*3+xstrlen(cwd.url)+2);
    sprintf(res,"%lu:",(unsigned long)time(0));
-   url::encode_string(cwd,res+strlen(res),URL_PATH_UNSAFE);
+   if(!cwd.url)
+   {
+      url::encode_string(cwd,res+strlen(res),URL_PATH_UNSAFE);
+      if(!cwd.is_file && url::dir_needs_trailing_slash(s->GetProto()))
+	 strcat(res,"/");
+   }
+   else
+      strcat(res,cwd.url);
    super::Add(s->GetConnectURL(s->NO_PATH|s->NO_PASSWORD),res);
    modified=true;
 }
@@ -154,6 +168,8 @@ void History::Set(FileAccess *s,const char *cwd)
 void History::Save()
 {
    Close();
+   if(!file)
+      return;
    if(!modified)
       return;
    fd=open(file,O_RDWR|O_CREAT,0600);
@@ -193,10 +209,9 @@ void History::Save()
    lseek(fd,0,SEEK_SET);
 
 #ifdef HAVE_FTRUNCATE
-   ftruncate(fd,0);
-#else
-   close(open(file,O_WRONLY|O_TRUNC));
+   if(ftruncate(fd,0)==-1) // note the following statement
 #endif
+   close(open(file,O_WRONLY|O_TRUNC));
 
    full->Write(fd);
    fd=-1;   // Write closes file
