@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.cc,v 1.71 2005/03/09 09:24:03 lav Exp $ */
+/* $Id: misc.cc,v 1.80 2005/12/27 12:16:59 lav Exp $ */
 
 #include <config.h>
 #include "xmalloc.h"
@@ -266,7 +266,7 @@ void  truncate_file_tree(const char *dir)
       SignalHook::Ignore(SIGTSTP);
       SignalHook::Ignore(SIGQUIT);
       SignalHook::Ignore(SIGHUP);
-      execlp("rm","rm","-rf",dir,NULL);
+      execlp("rm","rm","-rf",dir,(char*)NULL);
       perror("execlp(rm)");
       fflush(stderr);
       _exit(1);
@@ -441,7 +441,7 @@ int guess_year(int month,int day,int hour,int minute)
    const struct tm &now=SMTask::now;
    int year=now.tm_year+1900;
    if(month     *32+        day
-    > now.tm_mon*32+now.tm_mday+1)
+    > now.tm_mon*32+now.tm_mday+6)
       year--;
    return year;
 }
@@ -459,8 +459,9 @@ const char *squeeze_file_name(const char *name,int w)
    static int buf_len;
    int mbflags=MBSW_ACCEPT_INVALID|MBSW_ACCEPT_UNPRINTABLE;
 
-   int name_width=mbswidth(name,mbflags);
+   name=url::remove_password(name);
 
+   int name_width=mbswidth(name,mbflags);
    if(name_width<=w)
       return name;
 
@@ -638,7 +639,8 @@ char *Subst(const char *txt, const subst_t *s)
 	    unsigned len;
 	    unsigned code;
 	    txt--;
-	    sscanf(txt,"%3o%n",&code,&len);
+	    if(sscanf(txt,"%3o%n",&code,&len)!=1)
+	       continue; // should never happen.
 	    ch=code;
 	    txt+=len;
 	    str[0]=ch;
@@ -878,3 +880,74 @@ base64_encode (const char *s, char *store, int length)
   /* ...and zero-terminate it.  */
   *p = '\0';
 }
+
+bool temporary_network_error(int err)
+{
+   switch(err)
+   {
+   case(ETIMEDOUT):
+#ifdef ECONNRESET
+   case(ECONNRESET):
+#endif
+   case(ECONNREFUSED):
+#ifdef EHOSTUNREACH
+   case(EHOSTUNREACH):
+#endif
+#ifdef EHOSTDOWN
+   case(EHOSTDOWN):
+#endif
+#ifdef ENETRESET
+   case(ENETRESET):
+#endif
+#ifdef ENETUNREACH
+   case(ENETUNREACH):
+#endif
+#ifdef ENETDOWN
+   case(ENETDOWN):
+#endif
+#ifdef ECONNABORTED
+   case(ECONNABORTED):
+#endif
+      return true;
+   }
+   return false;
+}
+
+const char *get_lftp_home()
+{
+   static char *home=NULL;
+
+   if(home)
+      return home;
+
+   home=getenv("LFTP_HOME");
+   if(!home)
+   {
+      home=getenv("HOME");
+      if(home)
+      {
+         char *tmp=(char*)malloc(strlen(home)+6+1);
+         sprintf(tmp,"%s/.lftp",home);
+         home=tmp;
+      }
+      else
+         return NULL;
+   }
+   else
+      home=xstrdup(home);
+
+   if(!*home)
+      return NULL;
+
+   mkdir(home,0755);
+   return home;
+}
+
+const char *memrchr(const char *buf,char c,size_t len)
+{
+   buf+=len;
+   while(len-->0)
+      if(*--buf==c)
+	 return buf;
+   return 0;
+}
