@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,16 +18,17 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: netrc.cc,v 1.8.2.1 2002/01/15 16:20:27 lav Exp $ */
+/* $Id: netrc.cc,v 1.13 2005/12/27 06:25:51 lav Exp $ */
 
 #include <config.h>
 
 #include "trio.h"
 #include <stdlib.h>
+#include <errno.h>
 #include "xstring.h"
 #include "netrc.h"
-#include "xalloca.h"
 #include "xmalloc.h"
+#include "log.h"
 
 NetRC::Entry::Entry(const char *h,const char *u,const char *p,const char *a)
 {
@@ -59,7 +60,7 @@ static bool comment(const char *s, FILE *f)
    return true;
 }
 
-NetRC::Entry *NetRC::LookupHost(const char *h)
+NetRC::Entry *NetRC::LookupHost(const char *h,const char *u)
 {
    char str[256];
    char chost[256]="";
@@ -72,10 +73,14 @@ NetRC::Entry *NetRC::LookupHost(const char *h)
    char *netrc=(char*)alloca(strlen(home)+8);
    sprintf(netrc,"%s/.netrc",home);
    FILE *f=fopen(netrc,"r");
-   bool found=false;
+   bool host_found=false;
+   bool user_found=false;
 
    if(f==NULL)
+   {
+      Log::global->Format(10,"notice: cannot open %s: %s\n",netrc,strerror(errno));
       return NULL;
+   }
 
    while(fscanf(f,"%255s",str)==1)
    {
@@ -102,27 +107,36 @@ NetRC::Entry *NetRC::LookupHost(const char *h)
       }
       if(!strcmp(str,"machine"))
       {
+	 if(host_found && user_found)
+	    break;
 	 if(fscanf(f,"%255s",str)!=1)
 	    break;
 	 strcpy(chost,str);
-	 if(!strcasecmp(chost,h))
-	    found=true;
+	 // reset data for new machine.
+	 cuser[0]=0;
+	 cpass[0]=0;
+	 cacct[0]=0;
+	 host_found=!strcasecmp(chost,h);
+	 user_found=false;
 	 continue;
       }
       if(!strcmp(str,"login"))
       {
 	 if(fscanf(f,"%255s",str)!=1)
 	    break;
-	 if(strcasecmp(chost,h) || cuser[0])
+	 if(strcasecmp(chost,h))
 	    continue;
 	 strcpy(cuser,str);
+	 cpass[0]=0;
+	 cacct[0]=0;
+	 user_found=(!u || !strcasecmp(cuser,u));
 	 continue;
       }
       if(!strcmp(str,"password"))
       {
 	 if(fscanf(f,"%255s",str)!=1)
 	    break;
-	 if(strcasecmp(chost,h) || cpass[0])
+	 if(strcasecmp(chost,h) || (u && strcasecmp(cuser,u)) || cpass[0])
 	    continue;
 	 strcpy(cpass,str);
 	 for(char *s=cpass; *s; s++)
@@ -131,7 +145,8 @@ NetRC::Entry *NetRC::LookupHost(const char *h)
 	    {
 	       int ch=0;
 	       int n=0;
-	       sscanf(s+1,"%3o%n",&ch,&n);
+	       if(sscanf(s+1,"%3o%n",&ch,&n)!=1)
+		  continue;
 	       if(ch==0)
 		  continue;
 	       *s=ch;
@@ -144,14 +159,17 @@ NetRC::Entry *NetRC::LookupHost(const char *h)
       {
 	 if(fscanf(f,"%255s",str)!=1)
 	    break;
-	 if(strcasecmp(chost,h) || cacct[0])
+	 if(strcasecmp(chost,h) || (u && strcasecmp(cuser,u)) || cacct[0])
 	    continue;
 	 strcpy(cacct,str);
 	 continue;
       }
    }
    fclose(f);
-   if(found)
+   if(host_found && user_found)
+   {
+      Log::global->Format(10,"found netrc entry: host=%s, user=%s, pass=%s, acct=%s\n",h,cuser,cpass,cacct);
       return new Entry(h,cuser[0]?cuser:0,cpass[0]?cpass:0,cacct[0]?cacct:0);
+   }
    return 0;
 }
