@@ -28,19 +28,23 @@ class ParsedURL
    char	 *memory;
 public:
    char	 *proto;
-   char  *user;
+   char	 *user;
    char	 *pass;
-   char  *host;
+   char	 *host;
    char	 *port;
-   char  *path;
+   char	 *path;
 
-   ParsedURL(const char *url,bool proto_required=false);
+   char  *orig_url;
+
+   ParsedURL(const char *url,bool proto_required=false,bool use_rfc1738=true);
    ~ParsedURL()
    {
       xfree(memory);
+      xfree(orig_url);
    }
 
-   void Combine(char *buf,const char *home=0);
+   // returns allocated memory
+   char *Combine(const char *home=0,bool use_rfc1738=true);
 };
 
 # define URL_UNSAFE " <>\"%{}|\\^[]`"
@@ -78,6 +82,10 @@ public:
       }
 
    static int path_index(const char *p);
+   static bool dir_needs_trailing_slash(const char *proto);
+   static bool find_password_pos(const char *url,int *start,int *len);
+   static const char *remove_password(const char *url);
+   static const char *hide_password(const char *url);
 };
 
 #endif//URL_H
